from typing import List, Optional

from anyscale._private.anyscale_client import AnyscaleClientInterface
from anyscale._private.sdk import sdk_docs
from anyscale._private.sdk.base_sdk import Timer
from anyscale.cli_logger import BlockLogger
from anyscale.project._private.project_sdk import PrivateProjectSDK
from anyscale.project.commands import (
    _ADD_COLLABORATORS_DOCSTRINGS,
    _ADD_COLLABORATORS_EXAMPLE,
    add_collaborators,
)
from anyscale.project.models import CreateProjectCollaborator


class ProjectSDK:
    def __init__(
        self,
        *,
        client: Optional[AnyscaleClientInterface] = None,
        logger: Optional[BlockLogger] = None,
        timer: Optional[Timer] = None,
    ):
        self._private_sdk = PrivateProjectSDK(client=client, logger=logger, timer=timer)

    @sdk_docs(
        doc_py_example=_ADD_COLLABORATORS_EXAMPLE,
        arg_docstrings=_ADD_COLLABORATORS_DOCSTRINGS,
    )
    def add_collaborators(  # noqa: F811
        self, cloud: str, project: str, collaborators: List[CreateProjectCollaborator],
    ) -> str:
        """Batch add collaborators to a project.
        """
        return self._private_sdk.add_collaborators(cloud, project, collaborators)



from typing import List

from anyscale._private.sdk.base_sdk import BaseSDK
from anyscale.client.openapi_client.models import (
    AdminCreateUser as AdminCreateUserModel,
)
from anyscale.user.models import AdminCreatedUser, AdminCreateUser


class PrivateUserSDK(BaseSDK):
    def admin_batch_create(
        self, admin_create_users: List[AdminCreateUser]
    ) -> List[AdminCreatedUser]:
        created_users = self.client.admin_batch_create_users(
            [
                AdminCreateUserModel(**admin_create_user.to_dict())
                for admin_create_user in admin_create_users
            ]
        )

        return [
            AdminCreatedUser(
                user_id=created_user.user_id,
                name=created_user.name,
                email=created_user.email,
                created_at=created_user.created_at,
                is_sso_user=created_user.is_sso_user,
                lastname=created_user.lastname,
                title=created_user.title,
            )
            for created_user in created_users
        ]



import click

import anyscale
from anyscale.cli_logger import BlockLogger
from anyscale.commands import command_examples
from anyscale.commands.util import AnyscaleCommand
from anyscale.user.models import AdminCreateUser, AdminCreateUsers


log = BlockLogger()  # CLI Logger


@click.group("user", help="Manage users.")
def user_cli() -> None:
    pass


@user_cli.command(
    name="batch-create",
    cls=AnyscaleCommand,
    example=command_examples.USER_BATCH_CREATE_EXAMPLE,
)
@click.option(
    "--users-file",
    "-f",
    required=True,
    type=str,
    help="Path to a YAML file that contains the information for user accounts to be created.",
)
def admin_batch_create(users_file: str,) -> None:
    """
    Batch create, as an admin, users without email verification.
    """
    log.info("Creating users...")

    create_users = AdminCreateUsers.from_yaml(users_file)

    try:
        created_users = anyscale.user.admin_batch_create(
            admin_create_users=[
                AdminCreateUser(**create_user)
                for create_user in create_users.create_users
            ]
        )
    except ValueError as e:
        log.error(f"Error creating users: {e}")
        return

    log.info(f"{len(created_users)} users created.")



from typing import List

from anyscale._private.sdk import sdk_command
from anyscale.cloud._private.cloud_sdk import PrivateCloudSDK
from anyscale.cloud.models import CreateCloudCollaborator


_CLOUD_SDK_SINGLETON_KEY = "cloud_sdk"

_ADD_COLLABORATORS_EXAMPLE = """
import anyscale
from anyscale.cloud.models import CloudPermissionLevel, CreateCloudCollaborator

anyscale.cloud.add_collaborators(
    cloud="cloud_name",
    collaborators=[
        CreateCloudCollaborator(
            email="test1@anyscale.com",
            permission_level=CloudPermissionLevel.WRITE,
        ),
        CreateCloudCollaborator(
            email="test2@anyscale.com",
            permission_level=CloudPermissionLevel.READONLY,
        ),
    ],
)
"""

_ADD_COLLABORATORS_ARG_DOCSTRINGS = {
    "cloud": "The cloud to add users to.",
    "collaborators": "The list of collaborators to add to the cloud.",
}


@sdk_command(
    _CLOUD_SDK_SINGLETON_KEY,
    PrivateCloudSDK,
    doc_py_example=_ADD_COLLABORATORS_EXAMPLE,
    arg_docstrings=_ADD_COLLABORATORS_ARG_DOCSTRINGS,
)
def add_collaborators(
    cloud: str, collaborators: List[CreateCloudCollaborator], *, _sdk: PrivateCloudSDK
) -> str:
    """Batch add collaborators to a cloud."""
    return _sdk.add_collaborators(cloud, collaborators)



from typing import List, Optional

from anyscale._private.sdk import sdk_command
from anyscale.llm.model._private.models_sdk import PrivateLLMModelsSDK
from anyscale.llm.model.models import DeletedFineTunedModel, FineTunedModel


_LLM_MODELS_SDK_SINGLETON_KEY = "llm_models_sdk"


_RETRIEVE_EXAMPLE = """
import anyscale

anyscale.llm.model.get(model_id="my-model-id")
anyscale.llm.model.get(job_ib="prodjob_123")
"""

_RETRIEVE_ARG_DOCSTRINGS = {
    "model_id": " ID of the finetuned model that is being retrieved.",
    "job_id": " ID of the Anyscale job corresponding to the fine-tuning run.",
}


_DELETE_EXAMPLE = """
import anyscale

anyscale.llm.model.delete(model_id="my-model-id")
"""

_DELETE_ARG_DOCSTRINGS = {"model_id": " ID of the finetuned model to delete."}

_LIST_EXAMPLE = """
import anyscale

anyscale.llm.model.list()
anyscale.llm.model.list(cloud_id="cld_123")
anyscale.llm.model.list(project_id="prj_123")
anyscale.llm.model.list(cloud_id="cld_123", project_id="prj_123")
anyscale.llm.model.list(project_id="prj_123", max_items=10)
"""

_LIST_ARG_DOCSTRINGS = {
    "cloud_id": "Cloud ID to filter by. If not specified, all models from all the clouds visible to the user (filtered optionally by `project_id`)  are listed.",
    "project_id": "Project ID to filter by. If not specified, all the models from all visible projects to the user (filtered optionally by `cloud_id`) are listed.",
    "max_items": "Maximum number of items to show in the list. By default, the 20 most recently created models are fetched.",
}


@sdk_command(
    _LLM_MODELS_SDK_SINGLETON_KEY,
    PrivateLLMModelsSDK,
    doc_py_example=_RETRIEVE_EXAMPLE,
    arg_docstrings=_RETRIEVE_ARG_DOCSTRINGS,
)
def get(
    *,
    model_id: Optional[str] = None,
    job_id: Optional[str] = None,
    _sdk: PrivateLLMModelsSDK,
) -> FineTunedModel:
    """Retrieves model card for a finetuned model."""
    return _sdk.get(model_id=model_id, job_id=job_id)


@sdk_command(
    _LLM_MODELS_SDK_SINGLETON_KEY,
    PrivateLLMModelsSDK,
    doc_py_example=_DELETE_EXAMPLE,
    arg_docstrings=_DELETE_ARG_DOCSTRINGS,
)
def delete(model_id: str, _sdk: PrivateLLMModelsSDK,) -> DeletedFineTunedModel:
    """Deletes a finetuned model. Requires owner permission for the corresponding Anyscale project."""
    return _sdk.delete(model_id)


@sdk_command(
    _LLM_MODELS_SDK_SINGLETON_KEY,
    PrivateLLMModelsSDK,
    doc_py_example=_LIST_EXAMPLE,
    arg_docstrings=_LIST_ARG_DOCSTRINGS,
)
def list(  # noqa: A001
    *,
    cloud_id: Optional[str] = None,
    project_id: Optional[str] = None,
    max_items: int = 20,
    _sdk: PrivateLLMModelsSDK,
) -> List[FineTunedModel]:
    """Lists fine-tuned models available to the user.

    By default, all models in all visible clouds under all visible projects to the user are listed. This is optionally filtered by `project_id` and/or `cloud_id`.
    """
    return _sdk.list(cloud_id=cloud_id, project_id=project_id, max_items=max_items)



from typing import Optional

from anyscale._private.anyscale_client import AnyscaleClientInterface
from anyscale._private.sdk.base_sdk import BaseSDK
from anyscale._private.sdk.timer import Timer
from anyscale.cli_logger import BlockLogger
from anyscale.client.openapi_client.models import FineTunedModel as APIFineTunedModel
from anyscale.llm.model.models import DeletedFineTunedModel, FineTunedModel


class PrivateLLMModelsSDK(BaseSDK):
    def __init__(
        self,
        *,
        logger: Optional[BlockLogger] = None,
        client: Optional[AnyscaleClientInterface] = None,
        timer: Optional[Timer] = None,
    ):
        super().__init__(logger=logger, client=client, timer=timer)

    def _parse_response_model_get(self, model: APIFineTunedModel) -> FineTunedModel:
        return FineTunedModel(
            id=model.id,
            base_model_id=model.base_model_id,
            cloud_id=model.cloud_id,
            # model.created_at is a datetime object, convert to unix timestamp
            created_at=int(model.created_at.timestamp()),
            # `creator` is a MiniUser object, just retrieve email
            creator=model.creator.email if model.creator is not None else None,
            ft_type=model.ft_type,
            generation_config=model.generation_config,
            job_id=model.job_id,
            project_id=model.project_id,
            storage_uri=model.storage_uri,
            workspace_id=model.workspace_id,
        )

    def list(
        self,
        *,
        cloud_id: Optional[str] = None,
        project_id: Optional[str] = None,
        max_items: int = 20,
    ):
        finetuned_models = self.client.list_finetuned_models(
            cloud_id, project_id, max_items
        )
        parsed_models = [
            self._parse_response_model_get(model) for model in finetuned_models
        ]
        return parsed_models

    def get(
        self, *, model_id: Optional[str] = None, job_id: Optional[str] = None
    ) -> FineTunedModel:
        model = self.client.get_finetuned_model(model_id, job_id)
        return self._parse_response_model_get(model)

    def delete(self, model_id) -> DeletedFineTunedModel:
        deleted_model = self.client.delete_finetuned_model(model_id).to_dict()
        deleted_model["deleted_at"] = int(deleted_model["deleted_at"].timestamp())
        return DeletedFineTunedModel(**deleted_model)



from typing import List

from anyscale._private.sdk import sdk_command
from anyscale.user._private.user_sdk import PrivateUserSDK
from anyscale.user.models import AdminCreatedUser, AdminCreateUser


_USER_SDK_SINGLETON_KEY = "user_sdk"

_ADMIN_BATCH_CREATE_EXAMPLE = """
import anyscale
from anyscale.user.models import AdminCreateUser

anyscale.user.admin_batch_create(
    [AdminCreateUser(
        name="name",
        email="test@anyscale.com",
        password="",
        is_sso_user=False,
        lastname="lastname",
        title="title",
    ),],
)
"""

_ADMIN_BATCH_CREATE_ARG_DOCSTRINGS = {
    "admin_create_users": "Users to be created by an admin.",
}


@sdk_command(
    _USER_SDK_SINGLETON_KEY,
    PrivateUserSDK,
    doc_py_example=_ADMIN_BATCH_CREATE_EXAMPLE,
    arg_docstrings=_ADMIN_BATCH_CREATE_ARG_DOCSTRINGS,
)
def admin_batch_create(
    admin_create_users: List[AdminCreateUser], *, _sdk: PrivateUserSDK
) -> List[AdminCreatedUser]:
    """Batch create, as an admin, users without email verification.
    """
    return _sdk.admin_batch_create(admin_create_users)



import copy
from typing import Any, Dict, Optional, Union

from anyscale._private.workload import WorkloadSDK
from anyscale.compute_config.models import ComputeConfig
from anyscale.sdk.anyscale_client.models import (
    AccessConfig,
    ApplyServiceModel,
    GrpcProtocolConfig,
    ProductionServiceV2VersionModel,
    Protocols,
    RayGCSExternalStorageConfig as APIRayGCSExternalStorageConfig,
    ServiceConfig as ExternalAPIServiceConfig,
    ServiceEventCurrentState,
    ServiceModel,
    TracingConfig as APITracingConfg,
)
from anyscale.service.models import (
    RayGCSExternalStorageConfig,
    ServiceConfig,
    ServiceLogMode,
    ServiceState,
    ServiceStatus,
    ServiceVersionStatus,
    TracingConfig,
)
from anyscale.utils.runtime_env import parse_requirements_file
from anyscale.utils.workspace_notification import (
    WorkspaceNotification,
    WorkspaceNotificationAction,
)


class PrivateServiceSDK(WorkloadSDK):
    SERVICE_VERSION_ID_TRUNCATED_LEN = 8

    def _override_application_runtime_envs(
        self,
        config: ServiceConfig,
        *,
        autopopulate_in_workspace: bool = True,
        cloud_id: Optional[str] = None,
        workspace_requirements_path: Optional[str] = None,
    ) -> ServiceConfig:
        """Overrides the runtime_env of each application in the config.

        Local directories specified in the 'working_dir' or 'py_modules' fields will be
        uploaded and replaced with the resulting remote URIs.

        Requirements files will be loaded and populated into the 'pip' field.

        If autopopulate_from_workspace is passed and this code is running inside a
        workspace, the following defaults will be applied:
            - 'working_dir' will be set to '.'.
            - 'pip' will be set to the workspace-managed requirements file.
        """
        new_applications = copy.deepcopy(config.applications)
        new_runtime_envs = [app.get("runtime_env", {}) for app in new_applications]

        new_runtime_envs = self.override_and_upload_local_dirs(
            new_runtime_envs,
            working_dir_override=config.working_dir,
            excludes_override=config.excludes,
            cloud_id=cloud_id,
            autopopulate_in_workspace=autopopulate_in_workspace,
            additional_py_modules=config.py_modules,
        )
        new_runtime_envs = self.override_and_load_requirements_files(
            new_runtime_envs,
            requirements_override=config.requirements,
            workspace_requirements_path=workspace_requirements_path,
        )
        new_runtime_envs = self.update_env_vars(
            new_runtime_envs, env_vars_updates=config.env_vars
        )

        for app, new_runtime_env in zip(new_applications, new_runtime_envs):
            if new_runtime_env:
                app["runtime_env"] = new_runtime_env

        return config.options(
            applications=new_applications,
            requirements=None,
            working_dir=None,
            excludes=None,
        )

    def _get_default_name(self) -> str:
        """Get a default name for the service.

        A default is currently only generated when running inside a workspace
        (from the workspace cluster name), so this function errors if called outside
        a workspace.
        """
        name = self.get_current_workspace_name()
        if name is None:
            raise ValueError(
                "A service name must be provided when running outside of a workspace."
            )

        self.logger.info(f"No name was specified, using default: '{name}'.")
        return name

    def _log_deployed_service_info(
        self, service: ServiceModel, *, canary_percent: Optional[int]
    ):
        """Log user-facing information about a deployed service."""
        version_id_info = "version ID: {version_id}".format(
            version_id=self._get_user_facing_service_version_id(
                service.canary_version
                if service.canary_version is not None
                else service.primary_version
            )
        )
        details = (
            "("
            + version_id_info
            + (
                ")"
                if canary_percent is None
                else f", target canary percent: {canary_percent})"
            )
        )

        message = f"Service '{service.name}' deployed {details}."
        self.logger.info(message)
        if self.client.inside_workspace():
            self.client.send_workspace_notification(
                WorkspaceNotification(
                    body=message,
                    action=WorkspaceNotificationAction(
                        type="navigate-service", title="View Service", value=service.id,
                    ),
                ),
            )

        self.logger.info(
            f"View the service in the UI: '{self.client.get_service_ui_url(service.id)}'"
        )
        self.logger.info(
            "Query the service once it's running using the following curl command (add the path you want to query):"
        )
        auth_token_header = (
            ""
            if service.auth_token is None
            else f'-H "Authorization: Bearer {service.auth_token}" '
        )
        self.logger.info(f"curl {auth_token_header}{service.base_url}/")

    def _build_ray_serve_config(self, config: ServiceConfig) -> Dict[str, Any]:
        ray_serve_config: Dict[str, Any] = {"applications": config.applications}
        if config.http_options:
            ray_serve_config["http_options"] = config.http_options
        if config.grpc_options:
            ray_serve_config["grpc_options"] = config.grpc_options
        if config.logging_config:
            ray_serve_config["logging_config"] = config.logging_config

        return ray_serve_config

    def _build_grpc_protocol_config(self, config: ServiceConfig) -> GrpcProtocolConfig:
        enabled = (
            config.grpc_options is not None
            and len(config.grpc_options.get("service_names", [])) > 0
        )
        service_names = (
            config.grpc_options.get("service_names", [])
            if config.grpc_options is not None
            else []
        )
        # Since config doesn't allow passing a port, keeping it as default port.
        return GrpcProtocolConfig(enabled=enabled, service_names=service_names)

    def _build_apply_service_model_for_in_place_update(  # noqa: PLR0912
        self,
        name: str,
        config: ServiceConfig,
        *,
        canary_percent: Optional[int] = None,
        max_surge_percent: Optional[int] = None,
        existing_service: Optional[ServiceModel] = None,
    ) -> ApplyServiceModel:
        """Build the ApplyServiceModel for an in_place update.

        in_place updates:
            - must be performed on an existing service.
            - cannot be performed while a rollout is active.
            - do not support rollout options (canary_percent and max_surge_percent).
            - do not support changing cluster-level config options (image, compute_config, etc.).

        If cluster-level options are provided, a warning will be logged and they will be ignored.
        """
        if canary_percent is not None:
            raise ValueError(
                "canary_percent cannot be specified when doing an in_place update."
            )
        if max_surge_percent is not None:
            raise ValueError(
                "max_surge_percent cannot be specified when doing an in_place update."
            )
        if existing_service is None:
            raise RuntimeError(
                "in_place can only be used to update running services, "
                f"but no service was found with name '{name}'."
            )
        if existing_service.current_state in {
            ServiceEventCurrentState.TERMINATING,
            ServiceEventCurrentState.TERMINATED,
        }:
            raise RuntimeError(
                "in_place can only be used to update running services, "
                f"but service '{name}' is terminated."
            )

        if existing_service.canary_version is not None:
            raise RuntimeError(
                "in_place updates cannot be used while a rollout is in progress. "
                "Complete the rollout or roll back first."
            )

        if any(
            [
                config.image_uri is not None,
                config.registry_login_secret is not None,
                config.containerfile is not None,
                config.compute_config is not None,
                config.ray_gcs_external_storage_config is not None,
                config.tracing_config is not None,
            ]
        ):
            self.logger.warning(
                "Cluster-level options such as image and compute config "
                "are ignored when performing an in_place update."
            )

        existing_config: ProductionServiceV2VersionModel = existing_service.primary_version
        query_auth_token_enabled = existing_service.auth_token is not None
        cloud_id = self.client.get_cloud_id(
            compute_config_id=existing_config.compute_config_id
        )
        config = self._override_application_runtime_envs(
            config,
            cloud_id=cloud_id,
            workspace_requirements_path=self.client.get_workspace_requirements_path(),
        )

        project_id = self.client.get_project_id(
            parent_cloud_id=cloud_id, name=config.project
        )
        return ApplyServiceModel(
            name=name,
            project_id=project_id,
            ray_serve_config=self._build_ray_serve_config(config),
            build_id=existing_config.build_id,
            compute_config_id=existing_config.compute_config_id,
            rollout_strategy="IN_PLACE",
            config=ExternalAPIServiceConfig(
                access=AccessConfig(use_bearer_token=query_auth_token_enabled),
                protocols=Protocols(grpc=self._build_grpc_protocol_config(config)),
                env_vars=config.env_vars,
            ),
            ray_gcs_external_storage_config=existing_config.ray_gcs_external_storage_config,
            tracing_config=existing_config.tracing_config,
        )

    def _build_apply_service_model_for_rollout(  # noqa: PLR0912
        self,
        name: str,
        config: ServiceConfig,
        *,
        canary_percent: Optional[int] = None,
        max_surge_percent: Optional[int] = None,
        existing_service: Optional[ServiceModel] = None,
    ) -> ApplyServiceModel:
        """Build the ApplyServiceModel for a rolling update."""

        build_id = None
        if config.containerfile is not None:
            build_id = self._image_sdk.build_image_from_containerfile(
                name=f"image-for-service-{name}",
                containerfile=self.get_containerfile_contents(config.containerfile),
                anonymous=True,
                ray_version=config.ray_version,
            )
        elif config.image_uri is not None:
            build_id = self._image_sdk.registery_image(
                image_uri=config.image_uri,
                registry_login_secret=config.registry_login_secret,
                ray_version=config.ray_version,
            )

        if self._image_sdk.enable_image_build_for_tracked_requirements:
            requirements_path_to_be_populated_in_runtime_env = None
            requirements_path = self.client.get_workspace_requirements_path()
            if requirements_path is not None:
                requirements = parse_requirements_file(requirements_path)
                if requirements:
                    build_id = self._image_sdk.build_image_from_requirements(
                        name=f"image-for-service-{name}",
                        base_build_id=self.client.get_default_build_id(),
                        requirements=requirements,
                    )
        else:
            requirements_path_to_be_populated_in_runtime_env = (
                self.client.get_workspace_requirements_path()
            )

        if build_id is None:
            build_id = self.client.get_default_build_id()

        compute_config_id, cloud_id = self.resolve_compute_config_and_cloud_id(
            compute_config=config.compute_config, cloud=config.cloud
        )

        project_id = self.client.get_project_id(
            parent_cloud_id=cloud_id, name=config.project
        )
        if (
            existing_service is not None
            and existing_service.primary_version is not None
        ):
            existing_cloud_id = self.client.get_cloud_id(
                compute_config_id=existing_service.primary_version.compute_config_id
            )
            if cloud_id != existing_cloud_id:
                new_cloud = self.client.get_cloud(cloud_id=cloud_id)
                assert new_cloud is not None
                existing_cloud = self.client.get_cloud(cloud_id=existing_cloud_id)
                assert existing_cloud is not None
                raise ValueError(
                    f"The cloud for a service cannot be changed once it's created. "
                    f"Service '{name}' was created on cloud '{existing_cloud.name}', but "
                    f"the provided config is for cloud '{new_cloud.name}'."
                )

        env_vars_from_workspace = self.client.get_workspace_env_vars()
        if env_vars_from_workspace:
            if config.env_vars:
                # the precedence should be cli > workspace
                env_vars_from_workspace.update(config.env_vars)
                config = config.options(env_vars=env_vars_from_workspace)
            else:
                config = config.options(env_vars=env_vars_from_workspace)

        config = self._override_application_runtime_envs(
            config,
            cloud_id=cloud_id,
            workspace_requirements_path=requirements_path_to_be_populated_in_runtime_env,
        )

        ray_gcs_external_storage_config = None
        if config.ray_gcs_external_storage_config is not None:
            assert isinstance(
                config.ray_gcs_external_storage_config, RayGCSExternalStorageConfig
            )
            ray_gcs_external_storage_config = APIRayGCSExternalStorageConfig(
                enable=config.ray_gcs_external_storage_config.enabled,
            )
            if config.ray_gcs_external_storage_config.address is not None:
                ray_gcs_external_storage_config.address = (
                    config.ray_gcs_external_storage_config.address
                )
            if config.ray_gcs_external_storage_config.certificate_path is not None:
                ray_gcs_external_storage_config.redis_certificate_path = (
                    config.ray_gcs_external_storage_config.certificate_path
                )

        tracing_config = None
        if config.tracing_config is not None:
            assert isinstance(config.tracing_config, TracingConfig)
            tracing_config = APITracingConfg(enabled=config.tracing_config.enabled,)
            if config.tracing_config.exporter_import_path is not None:
                tracing_config.exporter_import_path = (
                    config.tracing_config.exporter_import_path
                )
            if config.tracing_config.sampling_ratio is not None:
                tracing_config.sampling_ratio = config.tracing_config.sampling_ratio

        return ApplyServiceModel(
            name=name,
            project_id=project_id,
            ray_serve_config=self._build_ray_serve_config(config),
            build_id=build_id,
            compute_config_id=compute_config_id,
            canary_percent=canary_percent,
            max_surge_percent=max_surge_percent,
            rollout_strategy="ROLLOUT",
            config=ExternalAPIServiceConfig(
                access=AccessConfig(use_bearer_token=config.query_auth_token_enabled),
                protocols=Protocols(grpc=self._build_grpc_protocol_config(config)),
                env_vars=config.env_vars,
            ),
            ray_gcs_external_storage_config=ray_gcs_external_storage_config,
            tracing_config=tracing_config,
        )

    def deploy(
        self,
        config: ServiceConfig,
        *,
        in_place: bool = False,
        canary_percent: Optional[int] = None,
        max_surge_percent: Optional[int] = None,
    ) -> str:
        if not isinstance(in_place, bool):
            raise TypeError("in_place must be a bool.")

        if canary_percent is not None:
            if not isinstance(canary_percent, int):
                raise TypeError("canary_percent must be an int.")
            if canary_percent < 0 or canary_percent > 100:
                raise ValueError("canary_percent must be between 0 and 100.")

        if max_surge_percent is not None:
            if not isinstance(max_surge_percent, int):
                raise TypeError("max_surge_percent must be an int.")
            if max_surge_percent < 0 or max_surge_percent > 100:
                raise ValueError("max_surge_percent must be between 0 and 100.")

        name = config.name or self._get_default_name()
        existing_service: Optional[ServiceModel] = self.client.get_service(
            name=name, cloud=config.cloud, project=config.project
        )
        if existing_service is None:
            self.logger.info(f"Starting new service '{name}'.")
        elif existing_service.current_state == ServiceEventCurrentState.TERMINATED:
            self.logger.info(f"Restarting existing service '{name}'.")
        else:
            self.logger.info(f"Updating existing service '{name}'.")

        if in_place:
            model = self._build_apply_service_model_for_in_place_update(
                name,
                config,
                canary_percent=canary_percent,
                max_surge_percent=max_surge_percent,
                existing_service=existing_service,
            )
        else:
            model = self._build_apply_service_model_for_rollout(
                name,
                config,
                canary_percent=canary_percent,
                max_surge_percent=max_surge_percent,
                existing_service=existing_service,
            )

        service: ServiceModel = self.client.rollout_service(model)
        self._log_deployed_service_info(service, canary_percent=canary_percent)

        return service.id

    def _resolve_to_service_model(
        self,
        *,
        name: Optional[str] = None,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
    ) -> ServiceModel:
        if name is None:
            name = self._get_default_name()

        model: Optional[ServiceModel] = self.client.get_service(
            name=name, cloud=cloud, project=project
        )
        if model is None:
            raise RuntimeError(f"Service with name '{name}' was not found.")

        return model

    def rollback(
        self,
        name: Optional[str] = None,
        *,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
        max_surge_percent: Optional[int] = None,
    ) -> str:
        model = self._resolve_to_service_model(name=name, cloud=cloud, project=project)

        self.client.rollback_service(model.id, max_surge_percent=max_surge_percent)

        return model.id

    def terminate(
        self,
        name: Optional[str] = None,
        *,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
    ) -> str:
        model = self._resolve_to_service_model(name=name, cloud=cloud, project=project)

        self.client.terminate_service(model.id)

        return model.id

    def archive(
        self,
        name: Optional[str] = None,
        *,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
    ) -> str:
        model = self._resolve_to_service_model(name=name, cloud=cloud, project=project)

        self.client.archive_service(model.id)

        return model.id

    def _get_user_facing_service_version_id(
        self, model: ProductionServiceV2VersionModel
    ) -> str:
        # NOTE(edoakes): the "version ID" exposed in the UI and tagged in the metrics is
        # not actually the DB ID, but a truncated version of it. We should store this
        # in the DB to avoid breakages, but for now I'm copying the existing logic.
        return model.id[-self.SERVICE_VERSION_ID_TRUNCATED_LEN :]

    def _service_version_model_to_status(
        self,
        model: ProductionServiceV2VersionModel,
        *,
        service_name: str,
        project_id: str,
        query_auth_token_enabled: bool,
    ) -> ServiceVersionStatus:
        image_uri = self._image_sdk.get_image_uri_from_build_id(model.build_id)
        if image_uri is None:
            raise RuntimeError(f"Failed to get image URI for ID {model.build_id}.")
        image_build = self._image_sdk.get_image_build(model.build_id)
        if image_build is None:
            raise RuntimeError(f"Failed to get image build for ID {model.build_id}.")

        ray_gcs_external_storage_config = None
        if model.ray_gcs_external_storage_config is not None:
            ray_gcs_external_storage_config = RayGCSExternalStorageConfig(
                enabled=model.ray_gcs_external_storage_config.enable,
                address=model.ray_gcs_external_storage_config.address,
                certificate_path=model.ray_gcs_external_storage_config.redis_certificate_path,
            )

        project = self.client.get_project(project_id)
        compute_config = self.get_user_facing_compute_config(model.compute_config_id)
        tracing_config = None
        if model.tracing_config is not None:
            tracing_config = TracingConfig(
                enabled=model.tracing_config.enabled,
                exporter_import_path=model.tracing_config.exporter_import_path,
                sampling_ratio=model.tracing_config.sampling_ratio,
            )

        return ServiceVersionStatus(
            id=self._get_user_facing_service_version_id(model),
            state=model.current_state,
            # NOTE(edoakes): there is also a "current_weight" field but it does not match the UI.
            weight=model.weight,
            config=ServiceConfig(
                name=service_name,
                applications=model.ray_serve_config["applications"],
                image_uri=str(image_uri),
                compute_config=compute_config,
                registry_login_secret=image_build.registry_login_secret,
                query_auth_token_enabled=query_auth_token_enabled,
                http_options=model.ray_serve_config.get("http_options", None),
                grpc_options=model.ray_serve_config.get("grpc_options", None),
                logging_config=model.ray_serve_config.get("logging_config", None),
                ray_gcs_external_storage_config=ray_gcs_external_storage_config,
                cloud=compute_config.cloud
                if compute_config and isinstance(compute_config, ComputeConfig)
                else None,
                project=project.name
                if project is not None and project.name != "default"
                else None,
                tracing_config=tracing_config,
            ),
        )

    def _model_state_to_state(
        self, model_state: ServiceEventCurrentState,
    ) -> ServiceState:
        # If we add a new state to the backend, old clients may not recognize it.
        # Rather than erroring out and causing old code to crash, return UNKNOWN.
        state: ServiceState = ServiceState(ServiceState.UNKNOWN)
        try:
            state = ServiceState(model_state)
        except ValueError:
            self.logger.warning(
                f"Got unrecognized state: '{model_state}'. "
                "You likely need to update the 'anyscale' package. "
                "If you still see this message after upgrading, contact Anyscale support."
            )

        return state

    def _service_model_to_status(self, model: ServiceModel) -> ServiceStatus:
        # TODO(edoakes): for some reason the primary_version is populated
        # when the service is terminated. This should be fixed in the backend.
        is_terminated = model.current_state == ServiceEventCurrentState.TERMINATED

        # TODO(edoakes): this is currently only exposed at the service level in the API,
        # which means that the per-version `query_auth_token_enabled` field will lie if
        # it's changed.
        query_auth_token_enabled = model.auth_token is not None

        primary_version = None
        if not is_terminated and model.primary_version is not None:
            primary_version = self._service_version_model_to_status(
                model.primary_version,
                service_name=model.name,
                project_id=model.project_id,
                query_auth_token_enabled=query_auth_token_enabled,
            )

        canary_version = None
        if not is_terminated and model.canary_version is not None:
            canary_version = self._service_version_model_to_status(
                model.canary_version,
                service_name=model.name,
                project_id=model.project_id,
                query_auth_token_enabled=query_auth_token_enabled,
            )

        return ServiceStatus(
            id=model.id,
            name=model.name,
            state=self._model_state_to_state(model.current_state),
            query_url=model.base_url,
            query_auth_token=model.auth_token,
            primary_version=primary_version,
            canary_version=canary_version,
        )

    def status(
        self, name: str, *, cloud: Optional[str] = None, project: Optional[str] = None
    ) -> ServiceStatus:
        model = self._resolve_to_service_model(name=name, cloud=cloud, project=project)

        return self._service_model_to_status(model)

    def wait(
        self,
        name: str,
        *,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
        state: ServiceState,
        timeout_s: float,
        interval_s: float,
    ):
        if not isinstance(timeout_s, (int, float)):
            raise TypeError("timeout_s must be a float")
        if timeout_s <= 0:
            raise ValueError("timeout_s must be >= 0")

        if not isinstance(interval_s, (int, float)):
            raise TypeError("interval_s must be a float")
        if interval_s <= 0:
            raise ValueError("interval_s must be >= 0")

        def _get_curr_state() -> ServiceState:
            model = self._resolve_to_service_model(
                name=name, cloud=cloud, project=project
            )

            return self._model_state_to_state(model.current_state)

        curr_state = _get_curr_state()
        self.logger.info(
            f"Waiting for service '{name}' to reach target state {state}, currently in state: {curr_state}"
        )
        for _ in self.timer.poll(timeout_s=timeout_s, interval_s=interval_s):
            new_state = _get_curr_state()
            if new_state != curr_state:
                self.logger.info(
                    f"Service '{name}' transitioned from {curr_state} to {new_state}"
                )
                curr_state = new_state

            if curr_state == state:
                self.logger.info(f"Service '{name}' reached target state, exiting")
                break
        else:
            raise TimeoutError(
                f"Service '{name}' did not reach target state {state} within {timeout_s}s. Last seen state: {curr_state}."
            )

    def controller_logs(
        self,
        name: str,
        *,
        cloud: Optional[str] = None,
        project: Optional[str] = None,
        canary: bool = False,
        mode: Union[str, ServiceLogMode] = ServiceLogMode.TAIL,
        max_lines: int = 1000,
    ) -> str:
        model = self._resolve_to_service_model(name=name, cloud=cloud, project=project)
        head = mode == ServiceLogMode.HEAD

        if canary:
            if model.canary_version is None:
                raise ValueError(
                    f"Service '{name}' is not currently rolling out. There is no canary version."
                )

            return self.client.controller_logs_for_service_version(
                model.canary_version, head, max_lines
            )

        if model.primary_version is None:
            raise ValueError(
                f"Service '{name}' is not ready yet. Please try again later..."
            )

        return self.client.controller_logs_for_service_version(
            model.primary_version, head, max_lines
        )



from typing import Optional

from anyscale._private.sdk import sdk_command
from anyscale.image._private.image_sdk import PrivateImageSDK
from anyscale.image.models import ImageBuild


_IMAGE_SDK_SINGLETON_KEY = "image_sdk"

_BUILD_EXAMPLE = """
import anyscale

containerfile = '''
FROM anyscale/ray:2.21.0-py39
RUN pip install --no-cache-dir pandas
'''

image_uri: str = anyscale.image.build(containerfile, name="mycoolimage")
"""

_BUILD_ARG_DOCSTRINGS = {
    "name": "The name of the image.",
    "containerfile": "The content of the Containerfile.",
    "ray_version": "The version of Ray to use in the image",
}


@sdk_command(
    _IMAGE_SDK_SINGLETON_KEY,
    PrivateImageSDK,
    doc_py_example=_BUILD_EXAMPLE,
    arg_docstrings=_BUILD_ARG_DOCSTRINGS,
)
def build(
    containerfile: str,
    *,
    name: str,
    ray_version: Optional[str] = None,
    _sdk: PrivateImageSDK
) -> str:
    """Build an image from a Containerfile.

    Returns the URI of the image.
    """
    return _sdk.build_image_from_containerfile_with_image_uri(
        name, containerfile, ray_version=ray_version
    )


_GET_EXAMPLE = """
import anyscale

image_status = anyscale.image.get(name="mycoolimage")
"""

_GET_ARG_DOCSTRINGS = {
    "name": (
        "Get the details of an image.\n\n"
        "The name can contain an optional version, e.g., 'name:version'. "
        "If no version is provided, the latest one will be used.\n\n"
    )
}


@sdk_command(
    _IMAGE_SDK_SINGLETON_KEY,
    PrivateImageSDK,
    doc_py_example=_GET_EXAMPLE,
    arg_docstrings=_GET_ARG_DOCSTRINGS,
)
def get(*, name: str, _sdk: PrivateImageSDK) -> ImageBuild:
    """The name can contain an optional version tag, i.e., 'name:version'.

    If no version is provided, the latest one will be returned.
    """
    return _sdk.get(name)


_REGISTER_EXAMPLE = """
import anyscale

image_uri: str = anyscale.image.register("docker.io/myuser/myimage:v2", name="mycoolimage")
"""

_REGISTER_ARG_DOCSTRINGS = {
    "image_uri": "The URI of the BYOD image to register.",
    "name": "Name for the container image. If the name already exists, a new version will be built. Otherwise, a new container image will be created.",
    "ray_version": "The Ray version (X.Y.Z) specified for this image specified by either an image URI or a containerfile. If not provided, the latest Ray version will be used.",
    "registry_login_secret": "Name or identifier of the secret containing credentials to authenticate to the docker registry hosting the image.",  # pragma: allowlist secret
}


@sdk_command(
    _IMAGE_SDK_SINGLETON_KEY,
    PrivateImageSDK,
    doc_py_example=_REGISTER_EXAMPLE,
    arg_docstrings=_REGISTER_ARG_DOCSTRINGS,
)
def register(
    image_uri: str,
    *,
    name: str,
    ray_version: Optional[str] = None,
    registry_login_secret: Optional[str] = None,
    _sdk: PrivateImageSDK
) -> str:
    """
    Registers a BYOD image with a container image name.

    Returns the URI of the image.
    """
    return _sdk.register_byod_image_with_name(
        image_uri,
        registry_login_secret=registry_login_secret,
        ray_version=ray_version,
        name=name,
    )



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class AicaEndpointEvent(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'id': 'str',
        'service_event_id': 'str',
        'aica_endpoint_id': 'str',
        'timestamp': 'datetime',
        'event_type': 'AicaEndpointEventType',
        'level': 'AicaEndpointEventLevel',
        'scope': 'AicaEndpointScope',
        'message': 'str',
        'cluster_event_id': 'str',
        'cluster_state_transition_id': 'str',
        'serve_deployment_event_id': 'str',
        'model_name': 'str'
    }

    attribute_map = {
        'id': 'id',
        'service_event_id': 'service_event_id',
        'aica_endpoint_id': 'aica_endpoint_id',
        'timestamp': 'timestamp',
        'event_type': 'event_type',
        'level': 'level',
        'scope': 'scope',
        'message': 'message',
        'cluster_event_id': 'cluster_event_id',
        'cluster_state_transition_id': 'cluster_state_transition_id',
        'serve_deployment_event_id': 'serve_deployment_event_id',
        'model_name': 'model_name'
    }

    def __init__(self, id=None, service_event_id=None, aica_endpoint_id=None, timestamp=None, event_type=None, level=None, scope=None, message=None, cluster_event_id=None, cluster_state_transition_id=None, serve_deployment_event_id=None, model_name=None, local_vars_configuration=None):  # noqa: E501
        """AicaEndpointEvent - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._id = None
        self._service_event_id = None
        self._aica_endpoint_id = None
        self._timestamp = None
        self._event_type = None
        self._level = None
        self._scope = None
        self._message = None
        self._cluster_event_id = None
        self._cluster_state_transition_id = None
        self._serve_deployment_event_id = None
        self._model_name = None
        self.discriminator = None

        self.id = id
        if service_event_id is not None:
            self.service_event_id = service_event_id
        self.aica_endpoint_id = aica_endpoint_id
        self.timestamp = timestamp
        if event_type is not None:
            self.event_type = event_type
        self.level = level
        self.scope = scope
        if message is not None:
            self.message = message
        if cluster_event_id is not None:
            self.cluster_event_id = cluster_event_id
        if cluster_state_transition_id is not None:
            self.cluster_state_transition_id = cluster_state_transition_id
        if serve_deployment_event_id is not None:
            self.serve_deployment_event_id = serve_deployment_event_id
        if model_name is not None:
            self.model_name = model_name

    @property
    def id(self):
        """Gets the id of this AicaEndpointEvent.  # noqa: E501

        Id of the AICA endpoint event.  # noqa: E501

        :return: The id of this AicaEndpointEvent.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this AicaEndpointEvent.

        Id of the AICA endpoint event.  # noqa: E501

        :param id: The id of this AicaEndpointEvent.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def service_event_id(self):
        """Gets the service_event_id of this AicaEndpointEvent.  # noqa: E501

        Service event this AICA endpoint event is based off.  # noqa: E501

        :return: The service_event_id of this AicaEndpointEvent.  # noqa: E501
        :rtype: str
        """
        return self._service_event_id

    @service_event_id.setter
    def service_event_id(self, service_event_id):
        """Sets the service_event_id of this AicaEndpointEvent.

        Service event this AICA endpoint event is based off.  # noqa: E501

        :param service_event_id: The service_event_id of this AicaEndpointEvent.  # noqa: E501
        :type: str
        """

        self._service_event_id = service_event_id

    @property
    def aica_endpoint_id(self):
        """Gets the aica_endpoint_id of this AicaEndpointEvent.  # noqa: E501

        Id of AICA endpoint this event belongs to.  # noqa: E501

        :return: The aica_endpoint_id of this AicaEndpointEvent.  # noqa: E501
        :rtype: str
        """
        return self._aica_endpoint_id

    @aica_endpoint_id.setter
    def aica_endpoint_id(self, aica_endpoint_id):
        """Sets the aica_endpoint_id of this AicaEndpointEvent.

        Id of AICA endpoint this event belongs to.  # noqa: E501

        :param aica_endpoint_id: The aica_endpoint_id of this AicaEndpointEvent.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and aica_endpoint_id is None:  # noqa: E501
            raise ValueError("Invalid value for `aica_endpoint_id`, must not be `None`")  # noqa: E501

        self._aica_endpoint_id = aica_endpoint_id

    @property
    def timestamp(self):
        """Gets the timestamp of this AicaEndpointEvent.  # noqa: E501

        Inserted at time of service event this AICA endpoint event is based off.  # noqa: E501

        :return: The timestamp of this AicaEndpointEvent.  # noqa: E501
        :rtype: datetime
        """
        return self._timestamp

    @timestamp.setter
    def timestamp(self, timestamp):
        """Sets the timestamp of this AicaEndpointEvent.

        Inserted at time of service event this AICA endpoint event is based off.  # noqa: E501

        :param timestamp: The timestamp of this AicaEndpointEvent.  # noqa: E501
        :type: datetime
        """
        if self.local_vars_configuration.client_side_validation and timestamp is None:  # noqa: E501
            raise ValueError("Invalid value for `timestamp`, must not be `None`")  # noqa: E501

        self._timestamp = timestamp

    @property
    def event_type(self):
        """Gets the event_type of this AicaEndpointEvent.  # noqa: E501

        Type of AICA endpoint event.  # noqa: E501

        :return: The event_type of this AicaEndpointEvent.  # noqa: E501
        :rtype: AicaEndpointEventType
        """
        return self._event_type

    @event_type.setter
    def event_type(self, event_type):
        """Sets the event_type of this AicaEndpointEvent.

        Type of AICA endpoint event.  # noqa: E501

        :param event_type: The event_type of this AicaEndpointEvent.  # noqa: E501
        :type: AicaEndpointEventType
        """

        self._event_type = event_type

    @property
    def level(self):
        """Gets the level of this AicaEndpointEvent.  # noqa: E501

        Log level of AICA endpoint event.  # noqa: E501

        :return: The level of this AicaEndpointEvent.  # noqa: E501
        :rtype: AicaEndpointEventLevel
        """
        return self._level

    @level.setter
    def level(self, level):
        """Sets the level of this AicaEndpointEvent.

        Log level of AICA endpoint event.  # noqa: E501

        :param level: The level of this AicaEndpointEvent.  # noqa: E501
        :type: AicaEndpointEventLevel
        """
        if self.local_vars_configuration.client_side_validation and level is None:  # noqa: E501
            raise ValueError("Invalid value for `level`, must not be `None`")  # noqa: E501

        self._level = level

    @property
    def scope(self):
        """Gets the scope of this AicaEndpointEvent.  # noqa: E501

        Scope of AICA endpoint event.  # noqa: E501

        :return: The scope of this AicaEndpointEvent.  # noqa: E501
        :rtype: AicaEndpointScope
        """
        return self._scope

    @scope.setter
    def scope(self, scope):
        """Sets the scope of this AicaEndpointEvent.

        Scope of AICA endpoint event.  # noqa: E501

        :param scope: The scope of this AicaEndpointEvent.  # noqa: E501
        :type: AicaEndpointScope
        """
        if self.local_vars_configuration.client_side_validation and scope is None:  # noqa: E501
            raise ValueError("Invalid value for `scope`, must not be `None`")  # noqa: E501

        self._scope = scope

    @property
    def message(self):
        """Gets the message of this AicaEndpointEvent.  # noqa: E501


        :return: The message of this AicaEndpointEvent.  # noqa: E501
        :rtype: str
        """
        return self._message

    @message.setter
    def message(self, message):
        """Sets the message of this AicaEndpointEvent.


        :param message: The message of this AicaEndpointEvent.  # noqa: E501
        :type: str
        """

        self._message = message

    @property
    def cluster_event_id(self):
        """Gets the cluster_event_id of this AicaEndpointEvent.  # noqa: E501

        Cluster event id this AICA endpoint event is based off.   # noqa: E501

        :return: The cluster_event_id of this AicaEndpointEvent.  # noqa: E501
        :rtype: str
        """
        return self._cluster_event_id

    @cluster_event_id.setter
    def cluster_event_id(self, cluster_event_id):
        """Sets the cluster_event_id of this AicaEndpointEvent.

        Cluster event id this AICA endpoint event is based off.   # noqa: E501

        :param cluster_event_id: The cluster_event_id of this AicaEndpointEvent.  # noqa: E501
        :type: str
        """

        self._cluster_event_id = cluster_event_id

    @property
    def cluster_state_transition_id(self):
        """Gets the cluster_state_transition_id of this AicaEndpointEvent.  # noqa: E501

        Cluster state transition id this AICA endpoint event is based off.   # noqa: E501

        :return: The cluster_state_transition_id of this AicaEndpointEvent.  # noqa: E501
        :rtype: str
        """
        return self._cluster_state_transition_id

    @cluster_state_transition_id.setter
    def cluster_state_transition_id(self, cluster_state_transition_id):
        """Sets the cluster_state_transition_id of this AicaEndpointEvent.

        Cluster state transition id this AICA endpoint event is based off.   # noqa: E501

        :param cluster_state_transition_id: The cluster_state_transition_id of this AicaEndpointEvent.  # noqa: E501
        :type: str
        """

        self._cluster_state_transition_id = cluster_state_transition_id

    @property
    def serve_deployment_event_id(self):
        """Gets the serve_deployment_event_id of this AicaEndpointEvent.  # noqa: E501

        Serve deployment event id this AICA endpoint event is based off.   # noqa: E501

        :return: The serve_deployment_event_id of this AicaEndpointEvent.  # noqa: E501
        :rtype: str
        """
        return self._serve_deployment_event_id

    @serve_deployment_event_id.setter
    def serve_deployment_event_id(self, serve_deployment_event_id):
        """Sets the serve_deployment_event_id of this AicaEndpointEvent.

        Serve deployment event id this AICA endpoint event is based off.   # noqa: E501

        :param serve_deployment_event_id: The serve_deployment_event_id of this AicaEndpointEvent.  # noqa: E501
        :type: str
        """

        self._serve_deployment_event_id = serve_deployment_event_id

    @property
    def model_name(self):
        """Gets the model_name of this AicaEndpointEvent.  # noqa: E501

        Model name if this AICA endpoint event refers to a model event.   # noqa: E501

        :return: The model_name of this AicaEndpointEvent.  # noqa: E501
        :rtype: str
        """
        return self._model_name

    @model_name.setter
    def model_name(self, model_name):
        """Sets the model_name of this AicaEndpointEvent.

        Model name if this AICA endpoint event refers to a model event.   # noqa: E501

        :param model_name: The model_name of this AicaEndpointEvent.  # noqa: E501
        :type: str
        """

        self._model_name = model_name

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, AicaEndpointEvent):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, AicaEndpointEvent):
            return True

        return self.to_dict() != other.to_dict()



from typing import Optional

from anyscale._private.models.model_base import ListResponse
from anyscale._private.sdk import sdk_command_v2
from anyscale._private.sdk.base_sdk import BaseSDK
from anyscale.llm.dataset._private import docs
from anyscale.llm.dataset._private.models import Dataset


@sdk_command_v2(
    doc_py_example=docs.GET_PY_EXAMPLE, arg_docstrings=docs.GET_PY_ARG_DOCSTRINGS,
)
def get(
    name: str, version: Optional[int] = None, project: Optional[str] = None
) -> Dataset:
    """Retrieves metadata about a dataset.

    :param name: Name of the dataset.
    :param version: Version of the dataset. If a negative integer is provided, the dataset returned is this many versions back of the latest version. Default: Latest version.
    :param project: Name of the Anyscale project that the dataset belongs to. If not provided, all projects will be searched.

    Example usage:
    ```python
    dataset = anyscale.llm.dataset.get("my_first_dataset")
    print(f"Dataset name: '{dataset.name}'")  # Dataset name: 'my_first_dataset'

    # Get the second latest version of the dataset
    prev_dataset = anyscale.llm.dataset.get("my_first_dataset", version=-1)
    ```

    Return:
        Dataset: The `Dataset` object.
    """
    _sdk = BaseSDK()
    dataset = _sdk.client.get_dataset(name, version, project)
    return dataset


@sdk_command_v2(
    doc_py_example=docs.UPLOAD_PY_EXAMPLE, arg_docstrings=docs.UPLOAD_PY_ARG_DOCSTRINGS,
)
def upload(
    dataset_file: str,
    name: str,
    *,
    description: Optional[str] = None,
    cloud: Optional[str] = None,
    project: Optional[str] = None,
) -> Dataset:
    """Uploads a dataset, or a new version of a dataset, to your Anyscale cloud.

    :param dataset_file: Path to the dataset file to upload.
    :param name: Name of a new dataset, or an existing dataset, to upload a new version of.
    :param description: Description of the dataset version.
    :param cloud: Name of the Anyscale cloud to upload a new dataset to. If not provided, the default cloud will be used.
    :param project: Name of the Anyscale project to upload a new dataset to. If not provided, the default project of the cloud will be used.

    Example usage:
    ```python
    anyscale.llm.dataset.upload("path/to/my_first_dataset.jsonl", name="my_first_dataset")
    anyscale.llm.dataset.upload("my_dataset.jsonl", "second_dataset")
    anyscale.llm.dataset.upload("my_dataset2.jsonl", "second_dataset", description="added 3 lines")
    ```
    Return:
        Dataset: The `Dataset` object representing the uploaded dataset.

    NOTE:
    If you are uploading a new version, have run this from within an Anyscale workspace,
    and neither `cloud` nor `project` are provided, the cloud and project of the workspace will be used.
    """
    _sdk = BaseSDK()
    dataset = _sdk.client.upload_dataset(
        dataset_file, name, description, cloud, project,
    )
    return dataset


@sdk_command_v2(
    doc_py_example=docs.DOWNLOAD_PY_EXAMPLE,
    arg_docstrings=docs.DOWNLOAD_PY_ARG_DOCSTRINGS,
)
def download(
    name: str, version: Optional[int] = None, project: Optional[str] = None
) -> bytes:
    """Downloads a dataset from your Anyscale cloud.

    :param name: Name of the dataset to download.
    :param version: Version of the dataset to download. If a negative integer is provided, the dataset returned is this many versions back of the latest version. Default: Latest version.
    :param project: Name of the Anyscale project to download the dataset from. If not provided, all projects will be searched.

    Example usage:
    ```python
    dataset_contents: bytes = anyscale.llm.dataset.download("my_first_dataset.jsonl")
    jsonl_obj = [json.loads(line) for line in dataset_contents.decode().splitlines()]

    prev_dataset_contents = anyscale.llm.dataset.download("my_first_dataset.jsonl", version=-1)
    ```

    Returns:
        bytes: The contents of the dataset file.
    """
    _sdk = BaseSDK()
    dataset_bytes = _sdk.client.download_dataset(name, version, project)
    return dataset_bytes


@sdk_command_v2(
    doc_py_example=docs.LIST_PY_EXAMPLE, arg_docstrings=docs.LIST_PY_ARG_DOCSTRINGS
)
def list(  # noqa: A001
    *,
    # Pagination
    limit: Optional[int] = None,
    after: Optional[str] = None,  # Unique ID to start listing after
    # Filtering
    name_contains: Optional[str] = None,  # Substring in name, case insensitive
    cloud: Optional[str] = None,
    project: Optional[str] = None,
) -> ListResponse[Dataset]:
    """
    Lists datasets.

    :param limit: Maximum number of datasets to return. Default: 1000.
    :param after: ID of the dataset to start the listing from. If provided, the list will start from the dataset after this ID.
    :param name_contains: Filter datasets by name. If provided, only datasets with name containing this string will be returned.
    :param cloud: Name of the Anyscale cloud to search in. If not provided, all clouds will be searched.
    :param project: Name of the Anyscale project to search in. If not provided, all projects will be searched.

    Example usage:
    ```
    datasets = anyscale.llm.dataset.list(limit=10)
    for d in datasets:
        print(f"Dataset name: '{d.name}'")  # Prints 10 dataset names
    ```

    Returns:
        ListResponse[Dataset]: List of `Dataset` objects.
    """
    _sdk = BaseSDK()
    list_response = _sdk.client.list_datasets(
        limit=limit,
        after=after,
        name_contains=name_contains,
        cloud=cloud,
        project=project,
    )
    return list_response



# coding: utf-8

"""
    Anyscale API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from anyscale_client.configuration import Configuration


class JobQueueSpec(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'job_queue_name': 'str',
        'execution_mode': 'JobQueueExecutionMode',
        'compute_config_id': 'str',
        'cluster_environment_build_id': 'str',
        'max_concurrency': 'int',
        'idle_timeout_sec': 'int'
    }

    attribute_map = {
        'job_queue_name': 'job_queue_name',
        'execution_mode': 'execution_mode',
        'compute_config_id': 'compute_config_id',
        'cluster_environment_build_id': 'cluster_environment_build_id',
        'max_concurrency': 'max_concurrency',
        'idle_timeout_sec': 'idle_timeout_sec'
    }

    def __init__(self, job_queue_name=None, execution_mode=None, compute_config_id=None, cluster_environment_build_id=None, max_concurrency=1, idle_timeout_sec=None, local_vars_configuration=None):  # noqa: E501
        """JobQueueSpec - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._job_queue_name = None
        self._execution_mode = None
        self._compute_config_id = None
        self._cluster_environment_build_id = None
        self._max_concurrency = None
        self._idle_timeout_sec = None
        self.discriminator = None

        if job_queue_name is not None:
            self.job_queue_name = job_queue_name
        if execution_mode is not None:
            self.execution_mode = execution_mode
        if compute_config_id is not None:
            self.compute_config_id = compute_config_id
        if cluster_environment_build_id is not None:
            self.cluster_environment_build_id = cluster_environment_build_id
        if max_concurrency is not None:
            self.max_concurrency = max_concurrency
        self.idle_timeout_sec = idle_timeout_sec

    @property
    def job_queue_name(self):
        """Gets the job_queue_name of this JobQueueSpec.  # noqa: E501

        Optional user-provided identifier of the queue that could be subsequently used to reference the queue when submitting jobs. Note that name has to be unique within the project.  # noqa: E501

        :return: The job_queue_name of this JobQueueSpec.  # noqa: E501
        :rtype: str
        """
        return self._job_queue_name

    @job_queue_name.setter
    def job_queue_name(self, job_queue_name):
        """Sets the job_queue_name of this JobQueueSpec.

        Optional user-provided identifier of the queue that could be subsequently used to reference the queue when submitting jobs. Note that name has to be unique within the project.  # noqa: E501

        :param job_queue_name: The job_queue_name of this JobQueueSpec.  # noqa: E501
        :type: str
        """

        self._job_queue_name = job_queue_name

    @property
    def execution_mode(self):
        """Gets the execution_mode of this JobQueueSpec.  # noqa: E501

        Execution mode of the jobs submitted into the queue (one of: FIFO,LIFO,PRIORITY  # noqa: E501

        :return: The execution_mode of this JobQueueSpec.  # noqa: E501
        :rtype: JobQueueExecutionMode
        """
        return self._execution_mode

    @execution_mode.setter
    def execution_mode(self, execution_mode):
        """Sets the execution_mode of this JobQueueSpec.

        Execution mode of the jobs submitted into the queue (one of: FIFO,LIFO,PRIORITY  # noqa: E501

        :param execution_mode: The execution_mode of this JobQueueSpec.  # noqa: E501
        :type: JobQueueExecutionMode
        """

        self._execution_mode = execution_mode

    @property
    def compute_config_id(self):
        """Gets the compute_config_id of this JobQueueSpec.  # noqa: E501

        The id of the compute configuration that will be used to create cluster associated with the queue. Defaults to default compute config in the given project  # noqa: E501

        :return: The compute_config_id of this JobQueueSpec.  # noqa: E501
        :rtype: str
        """
        return self._compute_config_id

    @compute_config_id.setter
    def compute_config_id(self, compute_config_id):
        """Sets the compute_config_id of this JobQueueSpec.

        The id of the compute configuration that will be used to create cluster associated with the queue. Defaults to default compute config in the given project  # noqa: E501

        :param compute_config_id: The compute_config_id of this JobQueueSpec.  # noqa: E501
        :type: str
        """

        self._compute_config_id = compute_config_id

    @property
    def cluster_environment_build_id(self):
        """Gets the cluster_environment_build_id of this JobQueueSpec.  # noqa: E501

        The id of the cluster environment build that will be used to create cluster associated with the queue.  # noqa: E501

        :return: The cluster_environment_build_id of this JobQueueSpec.  # noqa: E501
        :rtype: str
        """
        return self._cluster_environment_build_id

    @cluster_environment_build_id.setter
    def cluster_environment_build_id(self, cluster_environment_build_id):
        """Sets the cluster_environment_build_id of this JobQueueSpec.

        The id of the cluster environment build that will be used to create cluster associated with the queue.  # noqa: E501

        :param cluster_environment_build_id: The cluster_environment_build_id of this JobQueueSpec.  # noqa: E501
        :type: str
        """

        self._cluster_environment_build_id = cluster_environment_build_id

    @property
    def max_concurrency(self):
        """Gets the max_concurrency of this JobQueueSpec.  # noqa: E501

        Max number of jobs to be run concurrently. Defaults to 1, ie running no more than 1 job at a time.  # noqa: E501

        :return: The max_concurrency of this JobQueueSpec.  # noqa: E501
        :rtype: int
        """
        return self._max_concurrency

    @max_concurrency.setter
    def max_concurrency(self, max_concurrency):
        """Sets the max_concurrency of this JobQueueSpec.

        Max number of jobs to be run concurrently. Defaults to 1, ie running no more than 1 job at a time.  # noqa: E501

        :param max_concurrency: The max_concurrency of this JobQueueSpec.  # noqa: E501
        :type: int
        """

        self._max_concurrency = max_concurrency

    @property
    def idle_timeout_sec(self):
        """Gets the idle_timeout_sec of this JobQueueSpec.  # noqa: E501

        Max period of time queue will be accepting new jobs, before being sealed off and its associated cluster being shutdown  # noqa: E501

        :return: The idle_timeout_sec of this JobQueueSpec.  # noqa: E501
        :rtype: int
        """
        return self._idle_timeout_sec

    @idle_timeout_sec.setter
    def idle_timeout_sec(self, idle_timeout_sec):
        """Sets the idle_timeout_sec of this JobQueueSpec.

        Max period of time queue will be accepting new jobs, before being sealed off and its associated cluster being shutdown  # noqa: E501

        :param idle_timeout_sec: The idle_timeout_sec of this JobQueueSpec.  # noqa: E501
        :type: int
        """
        if self.local_vars_configuration.client_side_validation and idle_timeout_sec is None:  # noqa: E501
            raise ValueError("Invalid value for `idle_timeout_sec`, must not be `None`")  # noqa: E501

        self._idle_timeout_sec = idle_timeout_sec

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, JobQueueSpec):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, JobQueueSpec):
            return True

        return self.to_dict() != other.to_dict()



# AUTOGENERATED - modify shared_anyscale_util in root directory to make changes
from typing import Tuple


def _number_value(s: str) -> int:
    try:
        return int(s)
    except ValueError:
        return 0


def _number_suffix_split(s: str) -> Tuple[str, str]:
    n = len(s)
    for i in range(n):
        if not s[n - 1 - i].isdigit():
            return s[: n - i], s[n - i :]
    return "", s  # all digits


def _number_prefix_split(s: str) -> Tuple[str, str]:
    n = len(s)
    for i in range(n):
        if not s[i].isdigit():
            return s[:i], s[i:]
    return s, ""  # all digits


def ray_semver_compare(a: str, b: str) -> int:  # noqa: PLR0911 PLR0912
    """
    A dumb implementation of semver comparison because ray versions
    don't actually follow the semver standard.


    The standard for semver versions is
    "{MAJOR}.{MINOR}.{PATCH}-{PRE_RELEASE_VERSION}," such as 2.7.0 or 2.7.0-oss
    for Ray versions. However, since we use hyphens in the codebase to split
    and extract information, our standard for Ray versions is
    "{MAJOR}.{MINOR}.{PATCH}{PRE_RELEASE_VERSION}," like 2.7.0oss
    without "-" between the patch and the pre-release version.

    Two versions are equal if they are the same string.

    Example:
        ray_semver_compare("2.7.0", "2.7.0optimized") == -1
        ray_semver_compare("2.0.1rc1", "2.0.1rc2") == -1
    """
    if a == b:
        return 0
    if a == "":
        return -1
    if b == "":
        return 1

    a_parts = a.split(".")
    b_parts = b.split(".")

    for i, a_part in enumerate(a_parts):
        if i >= len(b_parts):
            # No b has no part anymore, then a is greater.
            return 1

        b_part = b_parts[i]
        if a_part == b_part:
            continue

        if i == 0:
            # looks like ray2
            a_str, a_num_str = _number_suffix_split(a_part)
            b_str, b_num_str = _number_suffix_split(b_part)
            a_tuples0 = (a_str, _number_value(a_num_str), a_num_str, a_part)
            b_tuples0 = (b_str, _number_value(b_num_str), b_num_str, b_part)
            return 1 if a_tuples0 > b_tuples0 else -1
        a_num_str, a_str = _number_prefix_split(a_part)
        b_num_str, b_str = _number_prefix_split(b_part)
        a_tuples = (_number_value(a_num_str), a_num_str, a_str, a_part)
        b_tuples = (_number_value(b_num_str), b_num_str, b_str, b_part)
        return 1 if a_tuples > b_tuples else -1

    if len(a_parts) < len(b_parts):
        return -1
    return 1 if a > b else -1



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class WorkspaceDataplaneProxiedArtifacts(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'requirements': 'str',
        'skip_packages_tracking': 'str',
        'environment_variables': 'list[str]',
        'dockerfile': 'str'
    }

    attribute_map = {
        'requirements': 'requirements',
        'skip_packages_tracking': 'skip_packages_tracking',
        'environment_variables': 'environment_variables',
        'dockerfile': 'dockerfile'
    }

    def __init__(self, requirements=None, skip_packages_tracking=None, environment_variables=None, dockerfile=None, local_vars_configuration=None):  # noqa: E501
        """WorkspaceDataplaneProxiedArtifacts - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._requirements = None
        self._skip_packages_tracking = None
        self._environment_variables = None
        self._dockerfile = None
        self.discriminator = None

        if requirements is not None:
            self.requirements = requirements
        if skip_packages_tracking is not None:
            self.skip_packages_tracking = skip_packages_tracking
        if environment_variables is not None:
            self.environment_variables = environment_variables
        if dockerfile is not None:
            self.dockerfile = dockerfile

    @property
    def requirements(self):
        """Gets the requirements of this WorkspaceDataplaneProxiedArtifacts.  # noqa: E501

        The requirements.txt of the workspace.  # noqa: E501

        :return: The requirements of this WorkspaceDataplaneProxiedArtifacts.  # noqa: E501
        :rtype: str
        """
        return self._requirements

    @requirements.setter
    def requirements(self, requirements):
        """Sets the requirements of this WorkspaceDataplaneProxiedArtifacts.

        The requirements.txt of the workspace.  # noqa: E501

        :param requirements: The requirements of this WorkspaceDataplaneProxiedArtifacts.  # noqa: E501
        :type: str
        """

        self._requirements = requirements

    @property
    def skip_packages_tracking(self):
        """Gets the skip_packages_tracking of this WorkspaceDataplaneProxiedArtifacts.  # noqa: E501

        The .skip_packages_tracking of the workspace.  # noqa: E501

        :return: The skip_packages_tracking of this WorkspaceDataplaneProxiedArtifacts.  # noqa: E501
        :rtype: str
        """
        return self._skip_packages_tracking

    @skip_packages_tracking.setter
    def skip_packages_tracking(self, skip_packages_tracking):
        """Sets the skip_packages_tracking of this WorkspaceDataplaneProxiedArtifacts.

        The .skip_packages_tracking of the workspace.  # noqa: E501

        :param skip_packages_tracking: The skip_packages_tracking of this WorkspaceDataplaneProxiedArtifacts.  # noqa: E501
        :type: str
        """

        self._skip_packages_tracking = skip_packages_tracking

    @property
    def environment_variables(self):
        """Gets the environment_variables of this WorkspaceDataplaneProxiedArtifacts.  # noqa: E501

        The environment variables file of the workspace.  # noqa: E501

        :return: The environment_variables of this WorkspaceDataplaneProxiedArtifacts.  # noqa: E501
        :rtype: list[str]
        """
        return self._environment_variables

    @environment_variables.setter
    def environment_variables(self, environment_variables):
        """Sets the environment_variables of this WorkspaceDataplaneProxiedArtifacts.

        The environment variables file of the workspace.  # noqa: E501

        :param environment_variables: The environment_variables of this WorkspaceDataplaneProxiedArtifacts.  # noqa: E501
        :type: list[str]
        """

        self._environment_variables = environment_variables

    @property
    def dockerfile(self):
        """Gets the dockerfile of this WorkspaceDataplaneProxiedArtifacts.  # noqa: E501

        The Dockerfile of the workspace.  # noqa: E501

        :return: The dockerfile of this WorkspaceDataplaneProxiedArtifacts.  # noqa: E501
        :rtype: str
        """
        return self._dockerfile

    @dockerfile.setter
    def dockerfile(self, dockerfile):
        """Sets the dockerfile of this WorkspaceDataplaneProxiedArtifacts.

        The Dockerfile of the workspace.  # noqa: E501

        :param dockerfile: The dockerfile of this WorkspaceDataplaneProxiedArtifacts.  # noqa: E501
        :type: str
        """

        self._dockerfile = dockerfile

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, WorkspaceDataplaneProxiedArtifacts):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, WorkspaceDataplaneProxiedArtifacts):
            return True

        return self.to_dict() != other.to_dict()



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class OrganizationMarketingQuestions(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'workload': 'str',
        'modalities': 'str',
        'model_types': 'str',
        'similar_tools': 'str',
        'cluster_size': 'ClusterSize'
    }

    attribute_map = {
        'workload': 'workload',
        'modalities': 'modalities',
        'model_types': 'model_types',
        'similar_tools': 'similar_tools',
        'cluster_size': 'cluster_size'
    }

    def __init__(self, workload=None, modalities=None, model_types=None, similar_tools=None, cluster_size=None, local_vars_configuration=None):  # noqa: E501
        """OrganizationMarketingQuestions - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._workload = None
        self._modalities = None
        self._model_types = None
        self._similar_tools = None
        self._cluster_size = None
        self.discriminator = None

        if workload is not None:
            self.workload = workload
        if modalities is not None:
            self.modalities = modalities
        if model_types is not None:
            self.model_types = model_types
        if similar_tools is not None:
            self.similar_tools = similar_tools
        if cluster_size is not None:
            self.cluster_size = cluster_size

    @property
    def workload(self):
        """Gets the workload of this OrganizationMarketingQuestions.  # noqa: E501


        :return: The workload of this OrganizationMarketingQuestions.  # noqa: E501
        :rtype: str
        """
        return self._workload

    @workload.setter
    def workload(self, workload):
        """Sets the workload of this OrganizationMarketingQuestions.


        :param workload: The workload of this OrganizationMarketingQuestions.  # noqa: E501
        :type: str
        """

        self._workload = workload

    @property
    def modalities(self):
        """Gets the modalities of this OrganizationMarketingQuestions.  # noqa: E501


        :return: The modalities of this OrganizationMarketingQuestions.  # noqa: E501
        :rtype: str
        """
        return self._modalities

    @modalities.setter
    def modalities(self, modalities):
        """Sets the modalities of this OrganizationMarketingQuestions.


        :param modalities: The modalities of this OrganizationMarketingQuestions.  # noqa: E501
        :type: str
        """

        self._modalities = modalities

    @property
    def model_types(self):
        """Gets the model_types of this OrganizationMarketingQuestions.  # noqa: E501


        :return: The model_types of this OrganizationMarketingQuestions.  # noqa: E501
        :rtype: str
        """
        return self._model_types

    @model_types.setter
    def model_types(self, model_types):
        """Sets the model_types of this OrganizationMarketingQuestions.


        :param model_types: The model_types of this OrganizationMarketingQuestions.  # noqa: E501
        :type: str
        """

        self._model_types = model_types

    @property
    def similar_tools(self):
        """Gets the similar_tools of this OrganizationMarketingQuestions.  # noqa: E501


        :return: The similar_tools of this OrganizationMarketingQuestions.  # noqa: E501
        :rtype: str
        """
        return self._similar_tools

    @similar_tools.setter
    def similar_tools(self, similar_tools):
        """Sets the similar_tools of this OrganizationMarketingQuestions.


        :param similar_tools: The similar_tools of this OrganizationMarketingQuestions.  # noqa: E501
        :type: str
        """

        self._similar_tools = similar_tools

    @property
    def cluster_size(self):
        """Gets the cluster_size of this OrganizationMarketingQuestions.  # noqa: E501


        :return: The cluster_size of this OrganizationMarketingQuestions.  # noqa: E501
        :rtype: ClusterSize
        """
        return self._cluster_size

    @cluster_size.setter
    def cluster_size(self, cluster_size):
        """Sets the cluster_size of this OrganizationMarketingQuestions.


        :param cluster_size: The cluster_size of this OrganizationMarketingQuestions.  # noqa: E501
        :type: ClusterSize
        """

        self._cluster_size = cluster_size

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, OrganizationMarketingQuestions):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, OrganizationMarketingQuestions):
            return True

        return self.to_dict() != other.to_dict()



from typing import Optional

import click

from anyscale.commands import command_examples
from anyscale.commands.util import AnyscaleCommand
from anyscale.controllers.llm.models_controller import ModelsController
from anyscale.util import validate_non_negative_arg


@click.group("model", help="Finetuned models stored on your Anyscale cloud.")
def models_cli():
    pass


@models_cli.command(
    name="get",
    short_help="Retrieves information for a model in your Anyscale cloud.",
    cls=AnyscaleCommand,
    is_alpha=True,
    example=command_examples.LLM_MODELS_GET_EXAMPLE,
)
@click.option(
    "--model-id",
    required=False,
    type=str,
    default=None,
    help="ID for the model of interest",
)
@click.option(
    "--job-id",
    required=False,
    type=str,
    default=None,
    help="ID for the Anyscale job corresponding to the fine-tuning run",
)
def get_model(model_id: Optional[str], job_id: Optional[str]) -> None:
    """
    Gets the model card for the given model ID or corresponding job ID.

    Example usage:

        anyscale llm model get --model-id my-model-id

        anyscale llm model get --job-id job_123
    """
    ModelsController().get_model(model_id=model_id, job_id=job_id)


@models_cli.command(
    name="delete",
    short_help="Delete a fine-tuned model in your Anyscale cloud.",
    cls=AnyscaleCommand,
    is_alpha=True,
    example=command_examples.LLM_MODELS_DELETE_EXAMPLE,
)
@click.argument("model_id", required=True)
def delete_model(model_id: str) -> None:
    """
    Deletes the model for the given model ID. Requires owner permission for the corresponding Anyscale project.

    MODEL_ID = ID for the model of interest

    Example usage:

        anyscale llm model delete my-model-id
    """
    ModelsController().delete_model(model_id)


@models_cli.command(
    name="list",
    short_help="Lists fine-tuned models available to the user.",
    cls=AnyscaleCommand,
    is_alpha=True,
    example=command_examples.LLM_MODELS_LIST_EXAMPLE,
)
@click.option(
    "--cloud-id",
    required=False,
    type=str,
    help="Cloud ID to filter by. If not specified, all models from all visible clouds (filtered optionally by `project_id`) are listed.",
)
@click.option(
    "--project-id",
    required=False,
    type=str,
    help="Project ID to filter by. If not specified, all the models from all visible projects (filtered optionally by `cloud_id`) are listed.",
)
@click.option(
    "--max-items",
    required=False,
    type=int,
    default=20,
    help="Maximum number of items to show in the list. By default, the 20 most recently created models are fetched.",
    callback=validate_non_negative_arg,
)
def list_models(cloud_id: Optional[str], project_id: Optional[str], max_items: int):
    """
    Lists fine-tuned models available to the user.

    By default, all models in all visible clouds under all visible projects to the user are listed. This is optionally filtered by `project_id` and/or `cloud_id`.

    Example usage:

        anyscale llm model list

        anyscale llm model list --max-items 50

        anyscale llm model list --cloud-id cld_123

        anyscale llm model list --project-id prj_123

        anyscale llm model list --cloud-id cld_123 --project-id prj_123


    NOTE:
    If you are running this from within an Anyscale workspace, and neither `cloud_id` nor `project_id` are provided, the cloud and project of the workspace will be used.
    """
    controller = ModelsController()
    controller.list_models(
        cloud_id=cloud_id, project_id=project_id, max_items=max_items
    )



from typing import List

from anyscale._private.sdk.base_sdk import BaseSDK
from anyscale.client.openapi_client.models import (
    CreateCloudCollaborator as CreateCloudCollaboratorModel,
)
from anyscale.cloud.models import CreateCloudCollaborator


class PrivateCloudSDK(BaseSDK):
    def add_collaborators(
        self, cloud: str, collaborators: List[CreateCloudCollaborator]
    ) -> str:
        cloud_id = self.client.get_cloud_id(cloud_name=cloud, compute_config_id=None)

        return self.client.add_cloud_collaborators(
            cloud_id=cloud_id,
            collaborators=[
                CreateCloudCollaboratorModel(
                    email=collaborator.email,
                    permission_level=collaborator.permission_level.lower(),
                )
                for collaborator in collaborators
            ],
        )



from typing import List, Optional

from anyscale._private.anyscale_client import AnyscaleClientInterface
from anyscale._private.sdk import sdk_docs
from anyscale._private.sdk.base_sdk import Timer
from anyscale.cli_logger import BlockLogger
from anyscale.user._private.user_sdk import PrivateUserSDK
from anyscale.user.commands import (
    _ADMIN_BATCH_CREATE_ARG_DOCSTRINGS,
    _ADMIN_BATCH_CREATE_EXAMPLE,
    admin_batch_create,
)
from anyscale.user.models import AdminCreatedUser, AdminCreateUser


class UserSDK:
    def __init__(
        self,
        *,
        client: Optional[AnyscaleClientInterface] = None,
        logger: Optional[BlockLogger] = None,
        timer: Optional[Timer] = None,
    ):
        self._private_sdk = PrivateUserSDK(client=client, logger=logger, timer=timer)

    @sdk_docs(
        doc_py_example=_ADMIN_BATCH_CREATE_EXAMPLE,
        arg_docstrings=_ADMIN_BATCH_CREATE_ARG_DOCSTRINGS,
    )
    def admin_batch_create(  # noqa: F811
        self, admin_create_users: List[AdminCreateUser],
    ) -> List[AdminCreatedUser]:
        """Batch create, as an admin, users without email verification.
        """
        return self._private_sdk.admin_batch_create(admin_create_users)



# coding: utf-8

"""
    Managed Ray API

    No description provided (generated by Openapi Generator https://github.com/openapitools/openapi-generator)  # noqa: E501

    The version of the OpenAPI document: 0.1.0
    Generated by: https://openapi-generator.tech
"""


import pprint
import re  # noqa: F401

import six

from openapi_client.configuration import Configuration


class CloudResource(object):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """

    """
    Attributes:
      openapi_types (dict): The key is attribute name
                            and the value is attribute type.
      attribute_map (dict): The key is attribute name
                            and the value is json key in definition.
    """
    openapi_types = {
        'compute_stack': 'ComputeStack',
        'kubernetes_ingress_external_address': 'str',
        'kubernetes_namespaces': 'list[str]',
        'kubernetes_zones': 'list[str]',
        'kubernetes_dataplane_identity': 'str',
        'kubernetes_nfs_persistent_volume_claim': 'str',
        'cloud_storage_bucket_name': 'str',
        'cloud_storage_bucket_endpoint': 'str',
        'cloud_storage_bucket_region': 'str',
        'nfs_mount_targets': 'list[NFSMountTarget]',
        'nfs_mount_path': 'str',
        'aws_vpc_id': 'str',
        'aws_subnet_ids_with_availability_zones': 'list[SubnetIdWithAvailabilityZoneAWS]',
        'aws_iam_role_arns': 'list[str]',
        'aws_security_groups': 'list[str]',
        'aws_s3_id': 'str',
        'aws_efs_id': 'str',
        'aws_efs_mount_target_ip': 'str',
        'aws_cloudformation_stack_id': 'str',
        'gcp_vpc_id': 'str',
        'gcp_subnet_ids': 'list[str]',
        'memorydb_cluster_config': 'AWSMemoryDBClusterConfig',
        'id': 'str',
        'cloud_id': 'str'
    }

    attribute_map = {
        'compute_stack': 'compute_stack',
        'kubernetes_ingress_external_address': 'kubernetes_ingress_external_address',
        'kubernetes_namespaces': 'kubernetes_namespaces',
        'kubernetes_zones': 'kubernetes_zones',
        'kubernetes_dataplane_identity': 'kubernetes_dataplane_identity',
        'kubernetes_nfs_persistent_volume_claim': 'kubernetes_nfs_persistent_volume_claim',
        'cloud_storage_bucket_name': 'cloud_storage_bucket_name',
        'cloud_storage_bucket_endpoint': 'cloud_storage_bucket_endpoint',
        'cloud_storage_bucket_region': 'cloud_storage_bucket_region',
        'nfs_mount_targets': 'nfs_mount_targets',
        'nfs_mount_path': 'nfs_mount_path',
        'aws_vpc_id': 'aws_vpc_id',
        'aws_subnet_ids_with_availability_zones': 'aws_subnet_ids_with_availability_zones',
        'aws_iam_role_arns': 'aws_iam_role_arns',
        'aws_security_groups': 'aws_security_groups',
        'aws_s3_id': 'aws_s3_id',
        'aws_efs_id': 'aws_efs_id',
        'aws_efs_mount_target_ip': 'aws_efs_mount_target_ip',
        'aws_cloudformation_stack_id': 'aws_cloudformation_stack_id',
        'gcp_vpc_id': 'gcp_vpc_id',
        'gcp_subnet_ids': 'gcp_subnet_ids',
        'memorydb_cluster_config': 'memorydb_cluster_config',
        'id': 'id',
        'cloud_id': 'cloud_id'
    }

    def __init__(self, compute_stack=None, kubernetes_ingress_external_address=None, kubernetes_namespaces=None, kubernetes_zones=None, kubernetes_dataplane_identity=None, kubernetes_nfs_persistent_volume_claim=None, cloud_storage_bucket_name=None, cloud_storage_bucket_endpoint=None, cloud_storage_bucket_region=None, nfs_mount_targets=None, nfs_mount_path=None, aws_vpc_id=None, aws_subnet_ids_with_availability_zones=None, aws_iam_role_arns=None, aws_security_groups=None, aws_s3_id=None, aws_efs_id=None, aws_efs_mount_target_ip=None, aws_cloudformation_stack_id=None, gcp_vpc_id=None, gcp_subnet_ids=None, memorydb_cluster_config=None, id=None, cloud_id=None, local_vars_configuration=None):  # noqa: E501
        """CloudResource - a model defined in OpenAPI"""  # noqa: E501
        if local_vars_configuration is None:
            local_vars_configuration = Configuration()
        self.local_vars_configuration = local_vars_configuration

        self._compute_stack = None
        self._kubernetes_ingress_external_address = None
        self._kubernetes_namespaces = None
        self._kubernetes_zones = None
        self._kubernetes_dataplane_identity = None
        self._kubernetes_nfs_persistent_volume_claim = None
        self._cloud_storage_bucket_name = None
        self._cloud_storage_bucket_endpoint = None
        self._cloud_storage_bucket_region = None
        self._nfs_mount_targets = None
        self._nfs_mount_path = None
        self._aws_vpc_id = None
        self._aws_subnet_ids_with_availability_zones = None
        self._aws_iam_role_arns = None
        self._aws_security_groups = None
        self._aws_s3_id = None
        self._aws_efs_id = None
        self._aws_efs_mount_target_ip = None
        self._aws_cloudformation_stack_id = None
        self._gcp_vpc_id = None
        self._gcp_subnet_ids = None
        self._memorydb_cluster_config = None
        self._id = None
        self._cloud_id = None
        self.discriminator = None

        if compute_stack is not None:
            self.compute_stack = compute_stack
        if kubernetes_ingress_external_address is not None:
            self.kubernetes_ingress_external_address = kubernetes_ingress_external_address
        if kubernetes_namespaces is not None:
            self.kubernetes_namespaces = kubernetes_namespaces
        if kubernetes_zones is not None:
            self.kubernetes_zones = kubernetes_zones
        if kubernetes_dataplane_identity is not None:
            self.kubernetes_dataplane_identity = kubernetes_dataplane_identity
        if kubernetes_nfs_persistent_volume_claim is not None:
            self.kubernetes_nfs_persistent_volume_claim = kubernetes_nfs_persistent_volume_claim
        if cloud_storage_bucket_name is not None:
            self.cloud_storage_bucket_name = cloud_storage_bucket_name
        if cloud_storage_bucket_endpoint is not None:
            self.cloud_storage_bucket_endpoint = cloud_storage_bucket_endpoint
        if cloud_storage_bucket_region is not None:
            self.cloud_storage_bucket_region = cloud_storage_bucket_region
        if nfs_mount_targets is not None:
            self.nfs_mount_targets = nfs_mount_targets
        if nfs_mount_path is not None:
            self.nfs_mount_path = nfs_mount_path
        if aws_vpc_id is not None:
            self.aws_vpc_id = aws_vpc_id
        if aws_subnet_ids_with_availability_zones is not None:
            self.aws_subnet_ids_with_availability_zones = aws_subnet_ids_with_availability_zones
        if aws_iam_role_arns is not None:
            self.aws_iam_role_arns = aws_iam_role_arns
        if aws_security_groups is not None:
            self.aws_security_groups = aws_security_groups
        if aws_s3_id is not None:
            self.aws_s3_id = aws_s3_id
        if aws_efs_id is not None:
            self.aws_efs_id = aws_efs_id
        if aws_efs_mount_target_ip is not None:
            self.aws_efs_mount_target_ip = aws_efs_mount_target_ip
        if aws_cloudformation_stack_id is not None:
            self.aws_cloudformation_stack_id = aws_cloudformation_stack_id
        if gcp_vpc_id is not None:
            self.gcp_vpc_id = gcp_vpc_id
        if gcp_subnet_ids is not None:
            self.gcp_subnet_ids = gcp_subnet_ids
        if memorydb_cluster_config is not None:
            self.memorydb_cluster_config = memorydb_cluster_config
        self.id = id
        self.cloud_id = cloud_id

    @property
    def compute_stack(self):
        """Gets the compute_stack of this CloudResource.  # noqa: E501

        The compute stack associated with this cloud resource.  # noqa: E501

        :return: The compute_stack of this CloudResource.  # noqa: E501
        :rtype: ComputeStack
        """
        return self._compute_stack

    @compute_stack.setter
    def compute_stack(self, compute_stack):
        """Sets the compute_stack of this CloudResource.

        The compute stack associated with this cloud resource.  # noqa: E501

        :param compute_stack: The compute_stack of this CloudResource.  # noqa: E501
        :type: ComputeStack
        """

        self._compute_stack = compute_stack

    @property
    def kubernetes_ingress_external_address(self):
        """Gets the kubernetes_ingress_external_address of this CloudResource.  # noqa: E501

        On the Kubernetes compute stack, the address of the ingress used for serving traffic for Anyscale services.  # noqa: E501

        :return: The kubernetes_ingress_external_address of this CloudResource.  # noqa: E501
        :rtype: str
        """
        return self._kubernetes_ingress_external_address

    @kubernetes_ingress_external_address.setter
    def kubernetes_ingress_external_address(self, kubernetes_ingress_external_address):
        """Sets the kubernetes_ingress_external_address of this CloudResource.

        On the Kubernetes compute stack, the address of the ingress used for serving traffic for Anyscale services.  # noqa: E501

        :param kubernetes_ingress_external_address: The kubernetes_ingress_external_address of this CloudResource.  # noqa: E501
        :type: str
        """

        self._kubernetes_ingress_external_address = kubernetes_ingress_external_address

    @property
    def kubernetes_namespaces(self):
        """Gets the kubernetes_namespaces of this CloudResource.  # noqa: E501

        A list of Kubernetes namespaces associated with this cloud resource.  # noqa: E501

        :return: The kubernetes_namespaces of this CloudResource.  # noqa: E501
        :rtype: list[str]
        """
        return self._kubernetes_namespaces

    @kubernetes_namespaces.setter
    def kubernetes_namespaces(self, kubernetes_namespaces):
        """Sets the kubernetes_namespaces of this CloudResource.

        A list of Kubernetes namespaces associated with this cloud resource.  # noqa: E501

        :param kubernetes_namespaces: The kubernetes_namespaces of this CloudResource.  # noqa: E501
        :type: list[str]
        """

        self._kubernetes_namespaces = kubernetes_namespaces

    @property
    def kubernetes_zones(self):
        """Gets the kubernetes_zones of this CloudResource.  # noqa: E501

        On the Kubernetes compute stack, a list of zones to launch pods in.  # noqa: E501

        :return: The kubernetes_zones of this CloudResource.  # noqa: E501
        :rtype: list[str]
        """
        return self._kubernetes_zones

    @kubernetes_zones.setter
    def kubernetes_zones(self, kubernetes_zones):
        """Sets the kubernetes_zones of this CloudResource.

        On the Kubernetes compute stack, a list of zones to launch pods in.  # noqa: E501

        :param kubernetes_zones: The kubernetes_zones of this CloudResource.  # noqa: E501
        :type: list[str]
        """

        self._kubernetes_zones = kubernetes_zones

    @property
    def kubernetes_dataplane_identity(self):
        """Gets the kubernetes_dataplane_identity of this CloudResource.  # noqa: E501

        The IAM identity of the kubernetes manager which will be used for the validation during registration.  # noqa: E501

        :return: The kubernetes_dataplane_identity of this CloudResource.  # noqa: E501
        :rtype: str
        """
        return self._kubernetes_dataplane_identity

    @kubernetes_dataplane_identity.setter
    def kubernetes_dataplane_identity(self, kubernetes_dataplane_identity):
        """Sets the kubernetes_dataplane_identity of this CloudResource.

        The IAM identity of the kubernetes manager which will be used for the validation during registration.  # noqa: E501

        :param kubernetes_dataplane_identity: The kubernetes_dataplane_identity of this CloudResource.  # noqa: E501
        :type: str
        """

        self._kubernetes_dataplane_identity = kubernetes_dataplane_identity

    @property
    def kubernetes_nfs_persistent_volume_claim(self):
        """Gets the kubernetes_nfs_persistent_volume_claim of this CloudResource.  # noqa: E501

        The name of the persistent volume claim used to mount NFS into pods.  # noqa: E501

        :return: The kubernetes_nfs_persistent_volume_claim of this CloudResource.  # noqa: E501
        :rtype: str
        """
        return self._kubernetes_nfs_persistent_volume_claim

    @kubernetes_nfs_persistent_volume_claim.setter
    def kubernetes_nfs_persistent_volume_claim(self, kubernetes_nfs_persistent_volume_claim):
        """Sets the kubernetes_nfs_persistent_volume_claim of this CloudResource.

        The name of the persistent volume claim used to mount NFS into pods.  # noqa: E501

        :param kubernetes_nfs_persistent_volume_claim: The kubernetes_nfs_persistent_volume_claim of this CloudResource.  # noqa: E501
        :type: str
        """

        self._kubernetes_nfs_persistent_volume_claim = kubernetes_nfs_persistent_volume_claim

    @property
    def cloud_storage_bucket_name(self):
        """Gets the cloud_storage_bucket_name of this CloudResource.  # noqa: E501

        A fully qualified storage bucket name for cloud storage, e.g. s3://bucket-name, gs://bucket-name, or azure://bucket-name.  # noqa: E501

        :return: The cloud_storage_bucket_name of this CloudResource.  # noqa: E501
        :rtype: str
        """
        return self._cloud_storage_bucket_name

    @cloud_storage_bucket_name.setter
    def cloud_storage_bucket_name(self, cloud_storage_bucket_name):
        """Sets the cloud_storage_bucket_name of this CloudResource.

        A fully qualified storage bucket name for cloud storage, e.g. s3://bucket-name, gs://bucket-name, or azure://bucket-name.  # noqa: E501

        :param cloud_storage_bucket_name: The cloud_storage_bucket_name of this CloudResource.  # noqa: E501
        :type: str
        """

        self._cloud_storage_bucket_name = cloud_storage_bucket_name

    @property
    def cloud_storage_bucket_endpoint(self):
        """Gets the cloud_storage_bucket_endpoint of this CloudResource.  # noqa: E501

        An endpoint for cloud storage, e.g. used to override the default cloud storage scheme's endpoint (e.g. for S3, this would be passed to the AWS_ENDPOINT_URL environment variable).  # noqa: E501

        :return: The cloud_storage_bucket_endpoint of this CloudResource.  # noqa: E501
        :rtype: str
        """
        return self._cloud_storage_bucket_endpoint

    @cloud_storage_bucket_endpoint.setter
    def cloud_storage_bucket_endpoint(self, cloud_storage_bucket_endpoint):
        """Sets the cloud_storage_bucket_endpoint of this CloudResource.

        An endpoint for cloud storage, e.g. used to override the default cloud storage scheme's endpoint (e.g. for S3, this would be passed to the AWS_ENDPOINT_URL environment variable).  # noqa: E501

        :param cloud_storage_bucket_endpoint: The cloud_storage_bucket_endpoint of this CloudResource.  # noqa: E501
        :type: str
        """

        self._cloud_storage_bucket_endpoint = cloud_storage_bucket_endpoint

    @property
    def cloud_storage_bucket_region(self):
        """Gets the cloud_storage_bucket_region of this CloudResource.  # noqa: E501

        The region of the cloud storage bucket.  # noqa: E501

        :return: The cloud_storage_bucket_region of this CloudResource.  # noqa: E501
        :rtype: str
        """
        return self._cloud_storage_bucket_region

    @cloud_storage_bucket_region.setter
    def cloud_storage_bucket_region(self, cloud_storage_bucket_region):
        """Sets the cloud_storage_bucket_region of this CloudResource.

        The region of the cloud storage bucket.  # noqa: E501

        :param cloud_storage_bucket_region: The cloud_storage_bucket_region of this CloudResource.  # noqa: E501
        :type: str
        """

        self._cloud_storage_bucket_region = cloud_storage_bucket_region

    @property
    def nfs_mount_targets(self):
        """Gets the nfs_mount_targets of this CloudResource.  # noqa: E501

        A set of mount targets that should be used for mounting NFS (potentially one per zone, or just a single one to be used for all zone(s) that the cloud resource spans).  # noqa: E501

        :return: The nfs_mount_targets of this CloudResource.  # noqa: E501
        :rtype: list[NFSMountTarget]
        """
        return self._nfs_mount_targets

    @nfs_mount_targets.setter
    def nfs_mount_targets(self, nfs_mount_targets):
        """Sets the nfs_mount_targets of this CloudResource.

        A set of mount targets that should be used for mounting NFS (potentially one per zone, or just a single one to be used for all zone(s) that the cloud resource spans).  # noqa: E501

        :param nfs_mount_targets: The nfs_mount_targets of this CloudResource.  # noqa: E501
        :type: list[NFSMountTarget]
        """

        self._nfs_mount_targets = nfs_mount_targets

    @property
    def nfs_mount_path(self):
        """Gets the nfs_mount_path of this CloudResource.  # noqa: E501

        The path of the NFS server to mount (e.g. on GCP, this is the \"root_dir\" parameter).  # noqa: E501

        :return: The nfs_mount_path of this CloudResource.  # noqa: E501
        :rtype: str
        """
        return self._nfs_mount_path

    @nfs_mount_path.setter
    def nfs_mount_path(self, nfs_mount_path):
        """Sets the nfs_mount_path of this CloudResource.

        The path of the NFS server to mount (e.g. on GCP, this is the \"root_dir\" parameter).  # noqa: E501

        :param nfs_mount_path: The nfs_mount_path of this CloudResource.  # noqa: E501
        :type: str
        """

        self._nfs_mount_path = nfs_mount_path

    @property
    def aws_vpc_id(self):
        """Gets the aws_vpc_id of this CloudResource.  # noqa: E501

        The AWS vpc id of this cloud.  # noqa: E501

        :return: The aws_vpc_id of this CloudResource.  # noqa: E501
        :rtype: str
        """
        return self._aws_vpc_id

    @aws_vpc_id.setter
    def aws_vpc_id(self, aws_vpc_id):
        """Sets the aws_vpc_id of this CloudResource.

        The AWS vpc id of this cloud.  # noqa: E501

        :param aws_vpc_id: The aws_vpc_id of this CloudResource.  # noqa: E501
        :type: str
        """

        self._aws_vpc_id = aws_vpc_id

    @property
    def aws_subnet_ids_with_availability_zones(self):
        """Gets the aws_subnet_ids_with_availability_zones of this CloudResource.  # noqa: E501

        A list of AWS subnet IDs with its availability zone of this cloud.  # noqa: E501

        :return: The aws_subnet_ids_with_availability_zones of this CloudResource.  # noqa: E501
        :rtype: list[SubnetIdWithAvailabilityZoneAWS]
        """
        return self._aws_subnet_ids_with_availability_zones

    @aws_subnet_ids_with_availability_zones.setter
    def aws_subnet_ids_with_availability_zones(self, aws_subnet_ids_with_availability_zones):
        """Sets the aws_subnet_ids_with_availability_zones of this CloudResource.

        A list of AWS subnet IDs with its availability zone of this cloud.  # noqa: E501

        :param aws_subnet_ids_with_availability_zones: The aws_subnet_ids_with_availability_zones of this CloudResource.  # noqa: E501
        :type: list[SubnetIdWithAvailabilityZoneAWS]
        """

        self._aws_subnet_ids_with_availability_zones = aws_subnet_ids_with_availability_zones

    @property
    def aws_iam_role_arns(self):
        """Gets the aws_iam_role_arns of this CloudResource.  # noqa: E501

        A list of 1 or 2 AWS IAM role arns of this cloud (the IAM role Anyscale uses to access this cloud & the IAM role assigned to nodes in the cluster). In the VM compute stack, both are required; in the K8S compute stack, only the first is required.  # noqa: E501

        :return: The aws_iam_role_arns of this CloudResource.  # noqa: E501
        :rtype: list[str]
        """
        return self._aws_iam_role_arns

    @aws_iam_role_arns.setter
    def aws_iam_role_arns(self, aws_iam_role_arns):
        """Sets the aws_iam_role_arns of this CloudResource.

        A list of 1 or 2 AWS IAM role arns of this cloud (the IAM role Anyscale uses to access this cloud & the IAM role assigned to nodes in the cluster). In the VM compute stack, both are required; in the K8S compute stack, only the first is required.  # noqa: E501

        :param aws_iam_role_arns: The aws_iam_role_arns of this CloudResource.  # noqa: E501
        :type: list[str]
        """

        self._aws_iam_role_arns = aws_iam_role_arns

    @property
    def aws_security_groups(self):
        """Gets the aws_security_groups of this CloudResource.  # noqa: E501

        A list of AWS security groups of this cloud.  # noqa: E501

        :return: The aws_security_groups of this CloudResource.  # noqa: E501
        :rtype: list[str]
        """
        return self._aws_security_groups

    @aws_security_groups.setter
    def aws_security_groups(self, aws_security_groups):
        """Sets the aws_security_groups of this CloudResource.

        A list of AWS security groups of this cloud.  # noqa: E501

        :param aws_security_groups: The aws_security_groups of this CloudResource.  # noqa: E501
        :type: list[str]
        """

        self._aws_security_groups = aws_security_groups

    @property
    def aws_s3_id(self):
        """Gets the aws_s3_id of this CloudResource.  # noqa: E501

        The AWS S3 id of this cloud.  # noqa: E501

        :return: The aws_s3_id of this CloudResource.  # noqa: E501
        :rtype: str
        """
        return self._aws_s3_id

    @aws_s3_id.setter
    def aws_s3_id(self, aws_s3_id):
        """Sets the aws_s3_id of this CloudResource.

        The AWS S3 id of this cloud.  # noqa: E501

        :param aws_s3_id: The aws_s3_id of this CloudResource.  # noqa: E501
        :type: str
        """

        self._aws_s3_id = aws_s3_id

    @property
    def aws_efs_id(self):
        """Gets the aws_efs_id of this CloudResource.  # noqa: E501

        The AWS EFS id of this cloud.  # noqa: E501

        :return: The aws_efs_id of this CloudResource.  # noqa: E501
        :rtype: str
        """
        return self._aws_efs_id

    @aws_efs_id.setter
    def aws_efs_id(self, aws_efs_id):
        """Sets the aws_efs_id of this CloudResource.

        The AWS EFS id of this cloud.  # noqa: E501

        :param aws_efs_id: The aws_efs_id of this CloudResource.  # noqa: E501
        :type: str
        """

        self._aws_efs_id = aws_efs_id

    @property
    def aws_efs_mount_target_ip(self):
        """Gets the aws_efs_mount_target_ip of this CloudResource.  # noqa: E501

        The AWS EFS mount target IP of this cloud.  # noqa: E501

        :return: The aws_efs_mount_target_ip of this CloudResource.  # noqa: E501
        :rtype: str
        """
        return self._aws_efs_mount_target_ip

    @aws_efs_mount_target_ip.setter
    def aws_efs_mount_target_ip(self, aws_efs_mount_target_ip):
        """Sets the aws_efs_mount_target_ip of this CloudResource.

        The AWS EFS mount target IP of this cloud.  # noqa: E501

        :param aws_efs_mount_target_ip: The aws_efs_mount_target_ip of this CloudResource.  # noqa: E501
        :type: str
        """

        self._aws_efs_mount_target_ip = aws_efs_mount_target_ip

    @property
    def aws_cloudformation_stack_id(self):
        """Gets the aws_cloudformation_stack_id of this CloudResource.  # noqa: E501

        The AWS CloudFormation stack id of this cloud.  # noqa: E501

        :return: The aws_cloudformation_stack_id of this CloudResource.  # noqa: E501
        :rtype: str
        """
        return self._aws_cloudformation_stack_id

    @aws_cloudformation_stack_id.setter
    def aws_cloudformation_stack_id(self, aws_cloudformation_stack_id):
        """Sets the aws_cloudformation_stack_id of this CloudResource.

        The AWS CloudFormation stack id of this cloud.  # noqa: E501

        :param aws_cloudformation_stack_id: The aws_cloudformation_stack_id of this CloudResource.  # noqa: E501
        :type: str
        """

        self._aws_cloudformation_stack_id = aws_cloudformation_stack_id

    @property
    def gcp_vpc_id(self):
        """Gets the gcp_vpc_id of this CloudResource.  # noqa: E501

        The GCP vpc id of this cloud.  # noqa: E501

        :return: The gcp_vpc_id of this CloudResource.  # noqa: E501
        :rtype: str
        """
        return self._gcp_vpc_id

    @gcp_vpc_id.setter
    def gcp_vpc_id(self, gcp_vpc_id):
        """Sets the gcp_vpc_id of this CloudResource.

        The GCP vpc id of this cloud.  # noqa: E501

        :param gcp_vpc_id: The gcp_vpc_id of this CloudResource.  # noqa: E501
        :type: str
        """

        self._gcp_vpc_id = gcp_vpc_id

    @property
    def gcp_subnet_ids(self):
        """Gets the gcp_subnet_ids of this CloudResource.  # noqa: E501

        A list of GCP subnet IDs of this cloud.  # noqa: E501

        :return: The gcp_subnet_ids of this CloudResource.  # noqa: E501
        :rtype: list[str]
        """
        return self._gcp_subnet_ids

    @gcp_subnet_ids.setter
    def gcp_subnet_ids(self, gcp_subnet_ids):
        """Sets the gcp_subnet_ids of this CloudResource.

        A list of GCP subnet IDs of this cloud.  # noqa: E501

        :param gcp_subnet_ids: The gcp_subnet_ids of this CloudResource.  # noqa: E501
        :type: list[str]
        """

        self._gcp_subnet_ids = gcp_subnet_ids

    @property
    def memorydb_cluster_config(self):
        """Gets the memorydb_cluster_config of this CloudResource.  # noqa: E501

        The AWS memorydb cluster config of this cloud.  # noqa: E501

        :return: The memorydb_cluster_config of this CloudResource.  # noqa: E501
        :rtype: AWSMemoryDBClusterConfig
        """
        return self._memorydb_cluster_config

    @memorydb_cluster_config.setter
    def memorydb_cluster_config(self, memorydb_cluster_config):
        """Sets the memorydb_cluster_config of this CloudResource.

        The AWS memorydb cluster config of this cloud.  # noqa: E501

        :param memorydb_cluster_config: The memorydb_cluster_config of this CloudResource.  # noqa: E501
        :type: AWSMemoryDBClusterConfig
        """

        self._memorydb_cluster_config = memorydb_cluster_config

    @property
    def id(self):
        """Gets the id of this CloudResource.  # noqa: E501

        Server assigned unique identifier.  # noqa: E501

        :return: The id of this CloudResource.  # noqa: E501
        :rtype: str
        """
        return self._id

    @id.setter
    def id(self, id):
        """Sets the id of this CloudResource.

        Server assigned unique identifier.  # noqa: E501

        :param id: The id of this CloudResource.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and id is None:  # noqa: E501
            raise ValueError("Invalid value for `id`, must not be `None`")  # noqa: E501

        self._id = id

    @property
    def cloud_id(self):
        """Gets the cloud_id of this CloudResource.  # noqa: E501

        The ID of the cloud that this cloud resource belongs to.  # noqa: E501

        :return: The cloud_id of this CloudResource.  # noqa: E501
        :rtype: str
        """
        return self._cloud_id

    @cloud_id.setter
    def cloud_id(self, cloud_id):
        """Sets the cloud_id of this CloudResource.

        The ID of the cloud that this cloud resource belongs to.  # noqa: E501

        :param cloud_id: The cloud_id of this CloudResource.  # noqa: E501
        :type: str
        """
        if self.local_vars_configuration.client_side_validation and cloud_id is None:  # noqa: E501
            raise ValueError("Invalid value for `cloud_id`, must not be `None`")  # noqa: E501

        self._cloud_id = cloud_id

    def to_dict(self):
        """Returns the model properties as a dict"""
        result = {}

        for attr, _ in six.iteritems(self.openapi_types):
            value = getattr(self, attr)
            if isinstance(value, list):
                result[attr] = list(map(
                    lambda x: x.to_dict() if hasattr(x, "to_dict") else x,
                    value
                ))
            elif hasattr(value, "to_dict"):
                result[attr] = value.to_dict()
            elif isinstance(value, dict):
                result[attr] = dict(map(
                    lambda item: (item[0], item[1].to_dict())
                    if hasattr(item[1], "to_dict") else item,
                    value.items()
                ))
            else:
                result[attr] = value

        return result

    def to_str(self):
        """Returns the string representation of the model"""
        return pprint.pformat(self.to_dict())

    def __repr__(self):
        """For `print` and `pprint`"""
        return self.to_str()

    def __eq__(self, other):
        """Returns true if both objects are equal"""
        if not isinstance(other, CloudResource):
            return False

        return self.to_dict() == other.to_dict()

    def __ne__(self, other):
        """Returns true if both objects are not equal"""
        if not isinstance(other, CloudResource):
            return True

        return self.to_dict() != other.to_dict()



"""
This file holds all of the CLI commands for the "anyscale machine-pool" path.
"""

import json

import click
import tabulate
import yaml

from anyscale.client.openapi_client.models import (
    DescribeMachinePoolResponse,
    SchedulerInfo,
)
from anyscale.commands import command_examples
from anyscale.commands.util import AnyscaleCommand
from anyscale.controllers.machine_pool_controller import MachinePoolController


@click.group(
    "machine-pool", help="Commands to interact with machine pools in Anyscale.",
)
def machine_pool_cli() -> None:
    pass


@machine_pool_cli.command(
    name="create",
    help="Create a machine pool in Anyscale.",
    cls=AnyscaleCommand,
    example=command_examples.MACHINE_POOL_CREATE_EXAMPLE,
    is_beta=True,
)
@click.option(
    "--name",
    type=str,
    required=True,
    help="Provide a machine pool name (must be unique within an organization).",
)
@click.option(
    "--enable-rootless-dataplane-config",
    type=bool,
    default=False,
    required=False,
    help="Enable the dataplane rootless configuration for nodes running under this machine pool.",
)
def create_machine_pool(name: str, enable_rootless_dataplane_config: bool) -> None:
    machine_pool_controller = MachinePoolController()
    output = machine_pool_controller.create_machine_pool(
        machine_pool_name=name,
        enable_rootless_dataplane_config=enable_rootless_dataplane_config,
    )
    print(
        f"Machine pool {output.machine_pool.machine_pool_name} has been created successfully (ID {output.machine_pool.machine_pool_id})."
    )


@machine_pool_cli.command(
    name="update",
    help="Update a machine pool in Anyscale.",
    cls=AnyscaleCommand,
    example=command_examples.MACHINE_POOL_UPDATE_EXAMPLE,
    is_beta=True,
)
@click.option("--name", type=str, required=True, help="Provide a machine pool name.")
@click.option(
    "--spec-file",
    type=str,
    required=True,
    help="Provide a path to a specification file.",
)
def update_machine_pool(name: str, spec_file: str) -> None:
    machine_pool_controller = MachinePoolController()
    machine_pool_controller.update_machine_pool(
        machine_pool_name=name, spec_file=spec_file
    )
    print(f"Updated machine pool '{name}'.")


@machine_pool_cli.command(
    name="describe",
    help="Describe a machine pool in Anyscale.",
    cls=AnyscaleCommand,
    example=command_examples.MACHINE_POOL_DESCRIBE_EXAMPLE,
    is_beta=True,
)
@click.option("--name", type=str, required=True, help="Provide a machine pool name.")
@click.option(
    "--format",
    "format_",
    type=str,
    default="table",
    required=False,
    help="Output format (table, json).",
)
def describe(name: str, format_: str) -> None:
    machine_pool_controller = MachinePoolController()
    response: DescribeMachinePoolResponse = machine_pool_controller.describe_machine_pool(
        machine_pool_name=name
    )
    scheduler_info: SchedulerInfo = response.scheduler_info  # type: ignore
    if format_ == "json":
        print(json.dumps(scheduler_info.to_dict(), default=str))
    elif format_ == "table":
        machines_table = []
        columns = [
            "MACHINE ID",
            "TYPE",
            "PARTITION",
            "STATE",
            "WORKLOAD DETAILS",
            "WORKLOAD SCORE",
            "WORKLOAD START TIME",
            "CLOUD INSTANCE ID",
        ]
        for row in scheduler_info.machines:
            machines_table.append(
                [
                    row.machine_id,
                    row.machine_type,
                    row.partition,
                    row.allocation_state,
                    f"{row.workload_info.workload_type}/{row.workload_info.workload_name}/{row.workload_info.workload_cloud}"
                    if row.workload_info.workload_name
                    else "",
                    row.workload_score,
                    str(
                        row.workload_info.workload_start_time.astimezone().strftime(
                            "%m/%d/%Y %I:%M:%S %p %Z"
                        )
                    )
                    if row.workload_info.workload_name
                    else "",
                    row.cloud_instance_id,
                ]
            )

        # Sort by (type, partition, state, workload start time)
        machines_table.sort(key=lambda x: (x[1], x[2], x[3], x[6]))

        print("Machines:")
        print(
            tabulate.tabulate(
                machines_table, tablefmt="outline", headers=columns, stralign="left"
            )
        )

        requests_table = []
        columns = [
            "SIZE",
            "MACHINE TYPE",
            "WORKLOAD DETAILS",
            "WORKLOAD START TIME",
            "WORKLOAD CLOUD",
            "PARTITION SCORES",
        ]
        for row in scheduler_info.requests:
            requests_table.append(
                [
                    row.size,
                    row.machine_type,
                    f"{row.workload_info.workload_type}/{row.workload_info.workload_name}",
                    str(
                        row.workload_info.workload_start_time.astimezone().strftime(
                            "%m/%d/%Y %I:%M:%S %p %Z"
                        )
                    )
                    if row.workload_info.workload_name
                    else "",
                    row.workload_info.workload_cloud,
                    row.partition_scores,
                ]
            )

        # Sort by (machine type, workload start time, size)
        print("Requests:")
        print(
            tabulate.tabulate(
                requests_table, tablefmt="outline", headers=columns, stralign="left"
            )
        )


@machine_pool_cli.command(
    name="delete",
    help="Delete a machine pool in Anyscale.",
    cls=AnyscaleCommand,
    example=command_examples.MACHINE_POOL_DELETE_EXAMPLE,
    is_beta=True,
)
@click.option("--name", type=str, required=True, help="Provide a machine pool name.")
def delete_machine_pool(name: str) -> None:
    machine_pool_controller = MachinePoolController()
    machine_pool_controller.delete_machine_pool(machine_pool_name=name)
    print(f"Deleted machine pool '{name}'.")


@machine_pool_cli.command(
    name="list",
    help="List machine pools in Anyscale.",
    cls=AnyscaleCommand,
    example=command_examples.MACHINE_POOL_LIST_EXAMPLE,
    is_beta=True,
)
@click.option(
    "--format",
    "format_",
    type=str,
    default="table",
    required=False,
    help="Output format (table, yaml).",
)
def list_machine_pools(format_: str) -> None:
    machine_pool_controller = MachinePoolController()
    result = machine_pool_controller.list_machine_pools()

    if format_ == "table":
        table = []
        columns = [
            "MACHINE POOL",
            "ID",
            "Clouds",
            "Rootless",
        ]
        for mp in result.machine_pools:
            table.append(
                [
                    mp.machine_pool_name,
                    mp.machine_pool_id,
                    "\n".join(mp.cloud_ids),
                    mp.enable_rootless_dataplane_config,
                ]
            )
        print(
            tabulate.tabulate(
                table, tablefmt="simple_grid", headers=columns, stralign="left"
            )
        )
    elif format_ == "yaml":
        rows = []
        for mp in result.machine_pools:
            rows.append(
                {
                    "machine_pool_name": mp.machine_pool_name,
                    "machine_pool_id": mp.machine_pool_id,
                    "cloud_ids": mp.cloud_ids,
                    "enable_rootless_dataplane_config": mp.enable_rootless_dataplane_config,
                    "spec": mp.spec,
                }
            )
        print(yaml.dump(data=rows, width=float("inf")))  # type: ignore
    else:
        raise click.ClickException(f"Invalid output format '{format}'.")


@machine_pool_cli.command(name="attach", help="Attach a machine pool to a cloud.")
@click.option("--name", type=str, required=True, help="Provide a machine pool name.")
@click.option("--cloud", type=str, required=True, help="Provide a cloud name.")
def attach_machine_pool_to_cloud(name: str, cloud: str) -> None:
    machine_pool_controller = MachinePoolController()
    machine_pool_controller.attach_machine_pool_to_cloud(
        machine_pool_name=name, cloud=cloud
    )
    print(f"Attached machine pool '{name}' to cloud '{cloud}'.")


@machine_pool_cli.command(name="detach", help="Detach a machine pool from a cloud.")
@click.option("--name", type=str, required=True, help="Provide a machine pool name.")
@click.option("--cloud", type=str, required=True, help="Provide a cloud name.")
def detach_machine_pool_from_cloud(name: str, cloud: str) -> None:
    machine_pool_controller = MachinePoolController()
    machine_pool_controller.detach_machine_pool_from_cloud(
        machine_pool_name=name, cloud=cloud
    )
    print(f"Detached machine pool '{name}' from cloud '{cloud}'.")



import logging
import shutil
from typing import Any

import requests


logger = logging.getLogger(__file__)


def copy_file(to_s3: bool, source: str, target: Any, download: bool) -> None:
    """Copy a file.

    The file source or target may be on S3.

    Args:
        to_s3 (bool): If this is True, then copy to/from S3, else the local
            disk. If this is True, then the file source or target will be a
            presigned URL to which GET or POST HTTP requests can be sent.
        source (str or S3 URL): Source file local pathname or S3 GET URL. If
            this is an S3 URL, target is assumed to be a local pathname.
        target (str or S3 URL): Target file local pathname or S3 URL with POST
            credentials. If this is an S3 URL, source is assumed to be a local
            pathname.
        download (bool): If this is True, then this will upload from source to
            target, else this will download.
    """
    try:
        if to_s3:
            if download:
                with open(target, "wb") as f:
                    response = requests.get(source)
                    for block in response.iter_content(1024):
                        f.write(block)
            else:
                with open(source, "rb") as f:
                    files = {"file": ("object", f)}
                    resp = requests.post(
                        target["url"], data=target["fields"], files=files
                    )
                    assert resp.ok, resp.text
        else:
            shutil.copyfile(source, target)
    except (OSError, AssertionError) as e:
        logger.warn("Failed to copy file %s , aborting", source)
        raise e



from typing import Optional

from anyscale._private.sdk import sdk_command
from anyscale.cli_logger import BlockLogger
from anyscale.schedule._private.schedule_sdk import PrivateScheduleSDK
from anyscale.schedule.models import ScheduleConfig, ScheduleState, ScheduleStatus


logger = BlockLogger()

_SCHEDULE_SDK_SINGLETON_KEY = "schedule_sdk"

_APPLY_EXAMPLE = """
import anyscale
from anyscale.job.models import JobConfig
from anyscale.schedule.models import ScheduleConfig

anyscale.schedule.apply(
    ScheduleConfig(
        cron_expression="0 0 * * * *",
        job_config=JobConfig(
            name="my-job",
            entrypoint="python main.py",
            working_dir=".",
        )
    )
)
"""

_APPLY_ARG_DOCSTRINGS = {"config": "The config options defining the schedule."}


@sdk_command(
    _SCHEDULE_SDK_SINGLETON_KEY,
    PrivateScheduleSDK,
    doc_py_example=_APPLY_EXAMPLE,
    arg_docstrings=_APPLY_ARG_DOCSTRINGS,
)
def apply(config: ScheduleConfig, *, _sdk: PrivateScheduleSDK) -> str:
    """Apply or update a schedule.

    Returns the id of the schedule.
    """
    return _sdk.apply(config)


_SET_STATE_EXAMPLE = """
import anyscale
from anyscale.schedule.models import ScheduleState

anyscale.schedule.set_state(
    id="my=schedule-id",
    state=ScheduleState.DISABLED,
)
"""

_SET_STATE_ARG_DOCSTRINGS = {
    "id": "The id of the schedule.",
    "name": "The name of the schedule.",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the job. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
    "state": "The state to set the schedule to.",
}


@sdk_command(
    _SCHEDULE_SDK_SINGLETON_KEY,
    PrivateScheduleSDK,
    doc_py_example=_SET_STATE_EXAMPLE,
    arg_docstrings=_SET_STATE_ARG_DOCSTRINGS,
)
def set_state(
    *,
    id: Optional[str] = None,  # noqa: A002
    name: Optional[str] = None,
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    state: ScheduleState,
    _sdk: PrivateScheduleSDK
) -> str:
    """Set the state of a schedule.

    Returns the id of the schedule.
    """
    return _sdk.set_state(id=id, name=name, cloud=cloud, project=project, state=state,)


_STATUS_EXAMPLE = """
import anyscale
anyscale.schedule.status(id="cronjob_yt389jvskwht9k2ygx7rj6iz62")
"""

_STATUS_ARG_DOCSTRINGS = {
    "id": "The id of the schedule.",
    "name": "The name of the schedule.",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the job. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
}


@sdk_command(
    _SCHEDULE_SDK_SINGLETON_KEY,
    PrivateScheduleSDK,
    doc_py_example=_STATUS_EXAMPLE,
    arg_docstrings=_STATUS_ARG_DOCSTRINGS,
)
def status(
    *,
    id: Optional[str] = None,  # noqa: A002
    name: Optional[str] = None,
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    _sdk: PrivateScheduleSDK
) -> ScheduleStatus:
    """Return the status of the schedule.
    """
    return _sdk.status(id=id, name=name, cloud=cloud, project=project)


_TRIGGER_EXAMPLE = """
import anyscale
anyscale.schedule.trigger(id="cronjob_yt389jvskwht9k2ygx7rj6iz62")
"""

_TRIGGER_ARG_DOCSTRINGS = {
    "id": "The id of the schedule.",
    "name": "The name of the schedule.",
    "cloud": "The Anyscale Cloud to run this workload on. If not provided, the organization default will be used (or, if running in a workspace, the cloud of the workspace).",
    "project": "Named project to use for the job. If not provided, the default project for the cloud will be used (or, if running in a workspace, the project of the workspace).",
}


@sdk_command(
    _SCHEDULE_SDK_SINGLETON_KEY,
    PrivateScheduleSDK,
    doc_py_example=_TRIGGER_EXAMPLE,
    arg_docstrings=_TRIGGER_ARG_DOCSTRINGS,
)
def trigger(
    *,
    id: Optional[str] = None,  # noqa: A002
    name: Optional[str] = None,
    cloud: Optional[str] = None,
    project: Optional[str] = None,
    _sdk: PrivateScheduleSDK
) -> str:
    """Trigger the execution of the schedule.
    """
    return _sdk.trigger(id=id, name=name, cloud=cloud, project=project)



from dataclasses import dataclass, field
from datetime import datetime
from typing import Optional

from anyscale._private.models.model_base import ModelBase
from anyscale.client.openapi_client.models import Dataset as InternalDataset
from anyscale.commands import command_examples
from anyscale.llm.dataset._private import docs


@dataclass(frozen=True)
class Dataset(ModelBase):
    """
    Metadata about a dataset, which is a file uploaded by a user to their Anyscale cloud.
    """

    __ignore_validation__ = True

    __doc_py_example__ = docs.GET_PY_EXAMPLE
    __doc_cli_example__ = command_examples.LLM_DATASET_GET_EXAMPLE

    id: str = field(metadata={"docstring": "The ID of the dataset."})
    name: str = field(metadata={"docstring": "The name of the dataset."})
    filename: str = field(
        metadata={"docstring": "The file name of the uploaded dataset."}
    )
    storage_uri: str = field(
        metadata={
            "docstring": "The URI at which the dataset is stored (eg. `s3://bucket/path/to/test.jsonl`)."
        }
    )
    version: int = field(metadata={"docstring": "The version of the dataset."})
    num_versions: int = field(
        metadata={"docstring": "Number of versions of the dataset."}
    )
    created_at: datetime = field(
        metadata={"docstring": "The time at which the dataset was uploaded."}
    )
    creator_id: str = field(
        metadata={"docstring": "The ID of the Anyscale user who uploaded the dataset."}
    )
    project_id: str = field(
        metadata={
            "docstring": "The ID of the Anyscale project that the dataset belongs to."
        }
    )
    cloud_id: str = field(
        metadata={
            "docstring": "The ID of the Anyscale cloud that the dataset belongs to."
        }
    )
    description: Optional[str] = field(
        default=None,
        metadata={"docstring": "The description of the current dataset version."},
    )

    @classmethod
    def parse_from_internal_model(cls, internal_model: InternalDataset) -> "Dataset":
        return Dataset(
            id=internal_model.id,
            name=internal_model.name,
            filename=internal_model.filename,
            storage_uri=internal_model.storage_uri,
            version=internal_model.version,
            num_versions=internal_model.num_versions,
            created_at=internal_model.created_at,
            creator_id=internal_model.creator_id,
            project_id=internal_model.project_id,
            cloud_id=internal_model.cloud_id,
            description=internal_model.description,
        )
